from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    if os_info['platform'] == "windows":
        return {'return': 1, 'error': 'Script not supported in windows yet!'}

    if env.get('CM_DATASET_LLAMA3_PATH', '') == '':
        env['CM_TMP_REQUIRE_DOWNLOAD'] = "yes"

    if env.get('CM_OUTDIRNAME', '') != '':
        env['CM_DOWNLOAD_PATH'] = env['CM_OUTDIRNAME']

    return {'return': 0}


def postprocess(i):

    env = i['env']

    if env.get('CM_TMP_REQUIRE_DOWNLOAD', '') == "yes":
        env['CM_DATASET_LLAMA3_PATH'] = os.path.join(
            env['CM_DATASET_LLAMA3_PATH'], env['CM_DATASET_FILE_NAME'])

    return {'return': 0}



# Author: Grigori Fursin
# Contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import os

from cmind.automation import Automation
from cmind import utils


class CAutomation(Automation):
    """
    Automation actions
    """

    ############################################################
    def __init__(self, cmind, automation_file):
        super().__init__(cmind, __file__)

    ############################################################
    def test(self, i):
        """
        Test automation

        Args:
          (CM input dict):

          (out) (str): if 'con', output to console

          automation (str): automation as CM string object

          parsed_automation (list): prepared in CM CLI or CM access function
                                    [ (automation alias, automation UID) ] or
                                    [ (automation alias, automation UID), (automation repo alias, automation repo UID) ]

          (artifact) (str): artifact as CM string object

          (parsed_artifact) (list): prepared in CM CLI or CM access function
                                    [ (artifact alias, artifact UID) ] or
                                    [ (artifact alias, artifact UID), (artifact repo alias, artifact repo UID) ]

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

          * Output from this automation action

        """

        import json
        print(json.dumps(i, indent=2))

        return {'return': 0}

    ############################################################
    def add(self, i):
        """
        Add CM script

        Args:
          (CM input dict):

          (out) (str): if 'con', output to console

          ...

        Returns:
          (CM return dict):

          * return (int): return code == 0 if no error and >0 if error
          * (error) (str): error string if return>0

        """

        self_automation = self.meta['alias'] + ',' + self.meta['uid']

        console = i.get('out') == 'con'

        artifact = i.get('artifact', '')
        if ':' not in artifact:
            artifact = 'mlcommons@ck:' + artifact

        j = artifact.find(':')
        name = artifact[j + 1:]

        # Check info
        if name == '':
            name = input('Enter your name: ').strip()
            if name == '':
                return {'return': 1, 'error': 'name can\'t be empty'}

            artifact += name

        # Check if doesn't exist
        r = self.cmind.access({'action': 'find',
                               'automation': self_automation,
                               'artifact': artifact})
        if r['return'] > 0:
            return r
        elif r['return'] == 0 and len(r['list']) > 0:
            return {'return': 1, 'error': 'CM artifact with name {} already exists in {}'.format(
                name, r['list'][0].path)}

        meta = i.get('meta', {})

        # Prepare meta
        org = meta.get('organization', '')
        if org == '':
            org = input('Enter your organization (optional): ').strip()

        url = input('Enter your webpage (optional): ').strip()

        tags = input(
            'Enter tags of your challenges separate by comma (you can add them later): ').strip()

        if meta.get('name', '') == '':
            meta = {'name': name}

        if org != '':
            meta['organization'] = org

        if url != '':
            meta['urls'] = [url]

        if tags != '':
            meta['ongoing'] = tags.split(',')

        # Add placeholder (use common action)
        i['out'] = 'con'
        # Avoid recursion - use internal CM add function to add the script
        # artifact
        i['common'] = True

        i['action'] = 'add'
        i['automation'] = self_automation
        i['artifact'] = artifact

        i['meta'] = meta

        print('')

        r = self.cmind.access(i)
        if r['return'] > 0:
            return r

        path = r['path']

        path2 = os.path.dirname(path)

        print('')
        print(
            'Please go to {}, add your directory to Git, commit and create PR:'.format(path2))
        print('')
        print('cd {}'.format(path2))
        print('git add "{}"'.format(name))
        print('git commit "{}"'.format(name))
        print('')
        print('Please join https://discord.gg/JjWNWXKxwT to discuss challenges!')
        print('Looking forward to your contributions!')

        return r



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    file_name = 'docker.exe' if os_info['platform'] == 'windows' else 'docker'
    env['FILE_NAME'] = file_name

    if 'CM_DOCKER_BIN_WITH_PATH' not in env:
        r = i['automation'].find_artifact({'file_name': file_name,
                                           'env': env,
                                           'os_info': os_info,
                                           'default_path_env_key': 'PATH',
                                           'detect_version': True,
                                           'env_path_key': 'CM_DOCKER_BIN_WITH_PATH',
                                           'run_script_input': i['run_script_input'],
                                           'recursion_spaces': recursion_spaces})
        if r['return'] > 0:
            if r['return'] == 16:
                run_file_name = "install"
                r = automation.run_native_script(
                    {'run_script_input': i['run_script_input'], 'env': env, 'script_name': run_file_name})
                if r['return'] > 0:
                    return r
            else:
                return r

    return {'return': 0}


def detect_version(i):
    r = i['automation'].parse_version({'match_text': r'[Docker|podman] version\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_DOCKER_VERSION',
                                       'which_env': i['env']})
    if r['return'] > 0:
        return r

    version = r['version']

    tool = "docker"

    if "podman" in r['string'].lower():
        tool = "podman"

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))
    return {'return': 0, 'version': version, "tool": tool}


def postprocess(i):
    env = i['env']

    r = detect_version(i)

    if r['return'] > 0:
        return r

    version = r['version']
    tool = r['tool']
    found_file_path = env['CM_DOCKER_BIN_WITH_PATH']

    found_path = os.path.dirname(found_file_path)
    env['CM_DOCKER_INSTALLED_PATH'] = found_path
    env['+PATH'] = [found_path]

    env['CM_DOCKER_CACHE_TAGS'] = 'version-' + version

    env['CM_DOCKER_VERSION'] = version

    env['CM_CONTAINER_TOOL'] = tool

    return {'return': 0, 'version': version}



def main():
    print('')

    # Import cmind to test break points
    import cmind.utils
    import os
    if os.environ.get('CM_TMP_DEBUG_UID', '') == 'f52670e5f3f345a2':
        cmind.utils.debug_here(
            __file__,
            port=5678,
            text='Debugging main.py!').breakpoint()

    print('HELLO WORLD from Python')

    x = 1
    print(x)


if __name__ == '__main__':
    main()



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    datadir = env.get('CM_DATA_DIR', os.getcwd())
    env['CM_DATA_DIR'] = datadir

    env['CM_BERT_CONFIG_DOWNLOAD_DIR'] = os.path.join(datadir, "phase1")
    env['CM_BERT_VOCAB_DOWNLOAD_DIR'] = os.path.join(datadir, "phase1")
    env['CM_BERT_DATA_DOWNLOAD_DIR'] = os.path.join(datadir, "download")

    env['CM_BERT_CHECKPOINT_DOWNLOAD_DIR'] = os.path.join(datadir, "phase1")

    if env.get("CM_TMP_VARIATION", "") == "nvidia":
        code_path = os.path.join(
            env['CM_GIT_REPO_CHECKOUT_PATH'],
            'NVIDIA',
            'benchmarks',
            'bert',
            'implementations',
            'pytorch-22.09')
        env['CM_RUN_DIR'] = code_path
    elif env.get("CM_TMP_VARIATION", "") == "reference":
        code_path = os.path.join(
            env['CM_MLPERF_TRAINING_SOURCE'],
            'language_model',
            'tensorflow',
            'bert',
            'cleanup_scripts')
        env['CM_RUN_DIR'] = code_path

    return {'return': 0}


def postprocess(i):

    env = i['env']

    data_dir = env['CM_DATA_DIR']
    env['CM_MLPERF_TRAINING_BERT_DATA_PATH'] = data_dir

    if env.get("CM_TMP_VARIATION", "") == "nvidia":
        env['CM_GET_DEPENDENT_CACHED_PATH'] = os.path.join(
            data_dir, "hdf5", "eval", "eval_all.hdf5")
    elif env.get("CM_TMP_VARIATION", "") == "reference":
        env['CM_GET_DEPENDENT_CACHED_PATH'] = os.path.join(
            data_dir, "tfrecords", "eval_10k")
        env['CM_MLPERF_TRAINING_BERT_TFRECORDS_PATH'] = os.path.join(
            data_dir, "tfrecords")

    env['CM_MLPERF_TRAINING_BERT_VOCAB_PATH'] = env['CM_BERT_VOCAB_FILE_PATH']
    env['CM_MLPERF_TRAINING_BERT_CONFIG_PATH'] = env['CM_BERT_CONFIG_FILE_PATH']

    return {'return': 0}



import pandas as pd
import sys

csv_file = sys.argv[1] if len(sys.argv) > 1 else "summary.csv"
md_file = sys.argv[2] if len(sys.argv) > 2 else "converted.md"

df = pd.read_csv(csv_file, engine='python')

with open(md_file, "w") as md:
    df.to_markdown(buf=md)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    script_path = i['artifact'].path

    automation = i['automation']

    cm = automation.cmind

    path = os.getcwd()

    url = env['CM_IPOL_SRC_URL']

    year = env.get('CM_IPOL_YEAR', '')
    number = env.get('CM_IPOL_NUMBER', '')

    url = url.replace(
        '{{CM_IPOL_YEAR}}',
        year).replace(
        '{{CM_IPOL_NUMBER}}',
        number)

    print('Downloading from {}'.format(url))

    r = cm.access({'action': 'download_file',
                   'automation': 'utils,dc2743f8450541e3',
                   'url': url})
    if r['return'] > 0:
        return r

    filename = r['filename']

    print('Unzipping file {}'.format(filename))

    r = cm.access({'action': 'unzip_file',
                   'automation': 'utils,dc2743f8450541e3',
                   'filename': filename})
    if r['return'] > 0:
        return r

    if os.path.isfile(filename):
        print('Removing file {}'.format(filename))
        os.remove(filename)

    # Get sub-directory from filename
    ff = os.path.splitext(filename)

    subdir = ff[0]

    env['CM_IPOL_PATH'] = os.path.join(path, subdir)

    # Applying patch
    cmd = 'patch -p0 < {}'.format(os.path.join(script_path,
                                  'patch', '20240127.patch'))

    print('Patching code: {}'.format(cmd))
    os.system(cmd)

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    if env.get('CM_TMP_ML_MODEL_PROVIDER', '') == 'nvidia':
        i['run_script_input']['script_name'] = 'run-nvidia'
        gpu_arch = int(
            float(
                env['CM_CUDA_DEVICE_PROP_GPU_COMPUTE_CAPABILITY']) *
            10)
        env['CM_GPU_ARCH'] = gpu_arch
        env['CM_TMP_REQUIRE_DOWNLOAD'] = 'no'
    else:
        path = env.get('LLAMA2_CHECKPOINT_PATH', '').strip()

        if env.get('CM_TMP_ML_MODEL_PROVIDER', '') == 'amd':
            env['CM_TMP_REQUIRE_DOWNLOAD'] = 'no'
            i['run_script_input']['script_name'] = 'run-amd'
            env['AMD_CODE_DIR'] = os.path.join(
                env['CM_MLPERF_INFERENCE_RESULTS_PATH'], 'closed', 'AMD', 'code')
            env['CM_LLAMA2_FINAL_SAFE_TENSORS_ROOT'] = os.getcwd()
            env['CM_LLAMA2_FINAL_SAFE_TENSORS_PATH'] = os.path.join(
                env['CM_LLAMA2_FINAL_SAFE_TENSORS_ROOT'], "llama.safetensors")
        else:
            if path == '' or not os.path.exists(path):
                env['CM_TMP_REQUIRE_DOWNLOAD'] = 'yes'

    return {'return': 0}


def postprocess(i):

    env = i['env']
    if env.get('LLAMA2_CHECKPOINT_PATH', '') == '':
        env['LLAMA2_CHECKPOINT_PATH'] = env['CM_ML_MODEL_PATH']
    else:
        env['CM_ML_MODEL_PATH'] = env['LLAMA2_CHECKPOINT_PATH']
    env['CM_ML_MODEL_LLAMA2_FILE_WITH_PATH'] = env['LLAMA2_CHECKPOINT_PATH']
    env['CM_GET_DEPENDENT_CACHED_PATH'] = env['CM_ML_MODEL_PATH']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    run_script_input = i['run_script_input']

    file_name = 'javac.exe' if os_info['platform'] == 'windows' else 'javac'

    cur_dir = os.getcwd()

    meta = i['meta']

    found = False
    install = env.get('CM_JAVAC_PREBUILT_INSTALL', '') in ['on', 'True', True]

    env_path_key = 'CM_JAVAC_BIN_WITH_PATH'

    # If not force install, search for artifact
    if not install:
        rr = i['automation'].find_artifact({'file_name': file_name,
                                           'env': env,
                                            'os_info': os_info,
                                            'default_path_env_key': 'PATH',
                                            'detect_version': True,
                                            'env_path_key': env_path_key,
                                            'run_script_input': i['run_script_input'],
                                            'hook': skip_path,
                                            'recursion_spaces': recursion_spaces})
        if rr['return'] == 0:
            found = True
        elif rr['return'] != 16:
            return rr

    # If not found or force install
    if not found or install:

        if os_info['platform'] == 'windows':
            env['CM_JAVAC_PREBUILT_HOST_OS'] = 'windows'
            env['CM_JAVAC_PREBUILT_EXT'] = '.zip'
        else:
            env['CM_JAVAC_PREBUILT_HOST_OS'] = 'linux'
            env['CM_JAVAC_PREBUILT_EXT'] = '.tar.gz'

        url = env['CM_JAVAC_PREBUILT_URL']
        filename = env['CM_JAVAC_PREBUILT_FILENAME']

        javac_prebuilt_version = env['CM_JAVAC_PREBUILT_VERSION']
        javac_prebuilt_build = env['CM_JAVAC_PREBUILT_BUILD']

        for key in ['CM_JAVAC_PREBUILT_VERSION',
                    'CM_JAVAC_PREBUILT_BUILD',
                    'CM_JAVAC_PREBUILT_HOST_OS',
                    'CM_JAVAC_PREBUILT_EXT']:
            url = url.replace('${' + key + '}', env[key])
            filename = filename.replace('${' + key + '}', env[key])

        env['CM_JAVAC_PREBUILT_URL'] = url
        env['CM_JAVAC_PREBUILT_FILENAME'] = filename

        print('')
        print(
            recursion_spaces +
            '    Downloading and installing prebuilt Java from {} ...'.format(
                url +
                filename))

        rr = automation.run_native_script(
            {'run_script_input': run_script_input, 'env': env, 'script_name': 'install-prebuilt'})
        if rr['return'] > 0:
            return rr

        target_path = os.path.join(
            cur_dir, 'jdk-' + java_prebuilt_version, 'bin')
        target_file = os.path.join(target_path, file_name)

        if not os.path.isfile(target_file):
            return {'return': 1,
                    'error': 'can\'t find target file {}'.format(target_file)}

        print('')
        print(
            recursion_spaces +
            '    Registering file {} ...'.format(target_file))

        env[env_path_key] = target_file

        if '+PATH' not in env:
            env['+PATH'] = []
        env['+PATH'].append(target_path)

    return {'return': 0}


def skip_path(i):

    # Avoid not complete path on Windows
    skip = False

    path = i['file']

    if 'javapath' in path:
        skip = True

    return {'return': 0, 'skip': skip}


def detect_version(i):

    r = i['automation'].parse_version({'match_text': r'javac\s*([\d.]+)',
                                       'group_number': 1,
                                       'env_key': 'CM_JAVAC_VERSION',
                                       'which_env': i['env'],
                                       'debug': True})
    if r['return'] > 0:
        return r

    version = r['version']

    print(i['recursion_spaces'] + '    Detected version: {}'.format(version))

    return {'return': 0, 'version': version}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']
    r = detect_version(i)
    if r['return'] > 0:
        return r

    version = env['CM_JAVAC_VERSION']
    env['CM_JAVAC_CACHE_TAGS'] = 'version-' + version

    found_file_path = env['CM_JAVAC_BIN_WITH_PATH']
    file_name = os.path.basename(found_file_path)
    file_path = os.path.dirname(found_file_path)

    env['CM_JAVAC_BIN'] = file_name

    if os_info['platform'] == 'windows':
        env['CM_JAVA_BIN'] = 'java.exe'
    else:
        env['CM_JAVA_BIN'] = 'java'

    env['CM_JAVA_BIN_WITH_PATH'] = os.path.join(file_path, env['CM_JAVA_BIN'])

    found_path = os.path.dirname(found_file_path)
    javac_home_path = os.path.dirname(found_path)

    env['JAVA_HOME'] = javac_home_path

    return {'return': 0, 'version': version}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess
import select
import sys
import grp


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    if os.geteuid() == 0:
        env['CM_SUDO'] = ''  # root user does not need sudo
        env['CM_SUDO_USER'] = "yes"
    else:
        if can_execute_sudo_without_password() or prompt_sudo() == 0:
            env['CM_SUDO_USER'] = "yes"
            env['CM_SUDO'] = 'sudo'

        else:
            env['CM_SUDO_USER'] = "no"
            env['CM_SUDO'] = ''

    return {'return': 0}


def can_execute_sudo_without_password():
    try:
        # Run a harmless command using sudo
        result = subprocess.run(
            # -n prevents sudo from prompting for a password
            ['sudo', '-n', 'true'],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )

        # Check the return code; if it's 0, sudo executed without needing a
        # password
        if result.returncode == 0:
            return True
        else:
            return False
    except Exception as e:
        print(f"An error occurred: {e}")
        return False


def reset_terminal():
    """Reset terminal to default settings."""
    subprocess.run(['stty', 'sane'])


def prompt_retry(timeout=10, default_retry=False):
    """Prompt the user with a yes/no question to retry the command, with a 10-second timeout."""

    # Check if we're in an interactive terminal
    if not sys.stdin.isatty():
        if default_retry:
            print(f"Non-interactive environment detected. Automatically retrying.")
        else:
            print(f"Non-interactive environment detected. Skipping retry.")
        return default_retry  # Automatically use the default in non-interactive terminals

    print(
        f"Timeout occurred. Do you want to try again? (y/n): ",
        end='',
        flush=True)

    # Use select to wait for user input with a timeout
    ready, _, _ = select.select([sys.stdin], [], [], timeout)

    if ready:
        answer = sys.stdin.readline().strip().lower()
        if answer in ['y', 'n']:
            return answer == 'y'  # Return True if 'y', False if 'n'
        print("\nInvalid input. Please enter 'y' or 'n'.")
        return prompt_retry(timeout)  # Re-prompt on invalid input
    else:
        print("\nNo input received in 10 seconds. Exiting.")
        return False  # No input within the timeout, so don't retry


def is_user_in_sudo_group():
    """Check if the current user is in the 'sudo' group."""
    try:
        sudo_group = grp.getgrnam('sudo').gr_mem
        return os.getlogin() in sudo_group
    except KeyError:
        # 'sudo' group doesn't exist (might be different on some systems)
        return False
    except Exception as e:
        print(f"Error checking sudo group: {str(e)}")
        return False


def prompt_sudo():
    if os.geteuid() != 0 and not is_user_in_sudo_group():  # No sudo required for root user

        # Prompt for the password
        import getpass

        if not os.isatty(sys.stdin.fileno()):
            print("Skipping password prompt - non-interactive terminal detected!")
            password = None
        else:
            password = getpass.getpass("Enter password (-1 to skip): ")

        # Check if the input is -1
        if password == "-1":
            print("Skipping sudo command.")
            return -1

        # Run the command with sudo, passing the password
        try:
            if password is None:
                r = subprocess.check_output(
                    ['sudo', '-S', 'echo'],
                    text=True,
                    stderr=subprocess.STDOUT,
                    timeout=15      # Capture the command output
                )
            else:
                r = subprocess.check_output(
                    ['sudo', '-S', 'echo'],
                    input=password + "\n",  # Pass the password to stdin
                    text=True,
                    stderr=subprocess.STDOUT,
                    timeout=15      # Capture the command output
                )
            return 0
        except subprocess.TimeoutExpired:
            print("Timedout")
            reset_terminal()  # Reset terminal to sane state
            if not prompt_retry():  # If the user chooses not to retry or times out
                return -1
        except subprocess.CalledProcessError as e:
            print(f"Command failed: {e.output.decode('utf-8')}")
            reset_terminal()  # Reset terminal in case of failure
            return -1
        except Exception as e:
            print(f"An error occurred: {str(e)}")
            reset_terminal()  # Always reset terminal after error
            return -1

    return 0


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    work_dir = env.get('CM_TUNE_TVM_MODEL_WORKDIR', '')

    if work_dir != '':
        if not os.path.exists(work_dir):
            raise FileNotFoundError(
                f"Error: the specified path \"{work_dir}\"does not exist")

        if not os.path.exists(f"{work_dir}/database_workload.json"):
            raise FileNotFoundError(
                "Error: the found workdir does not contain database_workload.json")

        if not os.path.exists(f"{work_dir}/database_tuning_record.json"):
            raise FileNotFoundError(
                "Error: the found workdir does not contain database_tuning_record.json")

        if env.get('CM_TUNE_TVM_MODEL', '') != '':
            print("The \"tune-model\" variation is selected, but at the same time the path to the existing \"work_dir\" is also specified. The compiled model will be based on the found existing \"work_dir\".")
            env["CM_TUNE_TVM_MODEL"] = "no"

    return {'return': 0}


def postprocess(i):

    env = i['env']

    env['CM_ML_MODEL_ORIGINAL_FILE_WITH_PATH'] = env['CM_ML_MODEL_FILE_WITH_PATH']
    env['CM_ML_MODEL_FILE'] = 'model-tvm.so'
    env['CM_ML_MODEL_PATH'] = os.path.join(os.getcwd())
    env['CM_ML_MODEL_FILE_WITH_PATH'] = os.path.join(
        os.getcwd(), env['CM_ML_MODEL_FILE'])
    env['CM_ML_MODEL_FRAMEWORK'] = "tvm-" + env['CM_ML_MODEL_FRAMEWORK']
    if 'CM_ML_MODEL_INPUT_SHAPES' in env.keys():
        env['CM_ML_MODEL_INPUT_SHAPES'] = env['CM_ML_MODEL_INPUT_SHAPES'].replace(
            "BATCH_SIZE", env['CM_ML_MODEL_MAX_BATCH_SIZE'])
    if 'CM_TVM_FRONTEND_FRAMEWORK' in env and env['CM_TVM_FRONTEND_FRAMEWORK'] == 'pytorch':
        env['CM_PREPROCESS_PYTORCH'] = 'yes'
    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import cmind
import os
import copy

base_path = {}
base_path_meta = {}

##########################################################################


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    return {'return': 0}

##########################################################################


def postprocess(i):

    env = i['env']

    return {'return': 0}


##########################################################################
def load_cfg(i):

    tags = i.get('tags', '')
    artifact = i.get('artifact', '')

    key = i.get('key', '')

    ii = {'action': 'find',
        'automation': 'cfg'}
    if artifact != '':
        ii['artifact'] = artifact
    elif tags != '':
        ii['tags'] = tags

    r = cmind.access(ii)
    if r['return'] > 0:
        return r

    lst = r['list']

    prune = i.get('prune', {})
    prune_key = prune.get('key', '')
    prune_key_uid = prune.get('key_uid', '')
    prune_uid = prune.get('uid', '')
    prune_list = prune.get('list', [])

    # Checking individual files inside CM entry
    selection = []

    if i.get('skip_files', False):
        for l in lst:
            meta = l.meta
            full_path = l.path

            meta['full_path'] = full_path

            add = True

            if prune_key != '' and prune_key_uid != '':
                if prune_key_uid not in meta.get(prune_key, []):
                    add = False

            if add:
                selection.append(meta)
    else:
        for l in lst:
            path = l.path

            main_meta = l.meta
            all_tags = main_meta.get('tags', [])

            files = os.listdir(path)

            for f in files:
                if key != '' and not f.startswith(key):
                    continue

                if f.startswith('_') or (not f.endswith(
                    '.json') and not f.endswith('.yaml')):
                    continue

                full_path = os.path.join(path, f)

                full_path_without_ext = full_path[:-5]

                r = cmind.utils.load_yaml_and_json(full_path_without_ext)
                if r['return'] > 0:
                    print('Warning: problem loading file {}'.format(full_path))
                else:
                    meta = r['meta']

                    # Check base
                    r = process_base(meta, full_path)
                    if r['return'] > 0:
                        return r
                    meta = r['meta']

                    uid = meta['uid']

                    # Check pruning
                    add = True

                    if len(prune) > 0:
                        if prune_uid != '' and uid != prune_uid:
                            add = False

                        if add and len(
                            prune_list) > 0 and uid not in prune_list:
                            add = False

                        if add and prune_key != '' and prune_key_uid != '' and prune_key_uid != meta.get(
                            prune_key, None):
                            add = False

                    if add:
                        meta['full_path'] = full_path

                        add_all_tags = copy.deepcopy(all_tags)

                        name = meta.get('name', '')
                        if name == '':
                            name = ' '.join(meta.get('tags', []))
                        name = name.strip()
                        meta['name'] = name

                        file_tags = meta.get('tags', '').strip()
                        if file_tags == '':
                            if name != '':
                                add_all_tags += [v.lower()
                                                         for v in name.split(' ')]
                        else:
                            add_all_tags += file_tags.split(',')

                        meta['all_tags'] = add_all_tags

                        meta['main_meta'] = main_meta

                        selection.append(meta)

    return {'return': 0, 'lst': lst, 'selection': selection}

##########################################################################


def process_base(meta, full_path):

    global base_path, base_path_meta

    _base = meta.get('_base', '')
    if _base != '':
        name = ''

        filename = _base
        full_path_base = os.path.dirname(full_path)

        if not filename.endswith('.yaml') and not filename.endswith('.json'):
            return {'return': 1, 'error': '_base file {} in {} must be .yaml or .json'.format(
                filename, full_path)}

        if ':' in _base:
            x = _base.split(':')
            name = x[0]

            full_path_base = base_path.get(name, '')
            if full_path_base == '':

                # Find artifact
                r = cmind.access({'action': 'find',
                                  'automation': 'cfg',
                                  'artifact': name})
                if r['return'] > 0:
                    return r

                lst = r['list']

                if len(lst) == 0:
                    if not os.path.isfile(path):
                        return {'return': 1, 'error': '_base artifact {} not found in {}'.format(
                            name, full_path)}

                full_path_base = lst[0].path

                base_path[name] = full_path_base

            filename = x[1]

        # Load base
        path = os.path.join(full_path_base, filename)

        if not os.path.isfile(path):
            return {'return': 1, 'error': '_base file {} not found in {}'.format(
                filename, full_path)}

        if path in base_path_meta:
            base = copy.deepcopy(base_path_meta[path])
        else:
            path_without_ext = path[:-5]

            r = cmind.utils.load_yaml_and_json(path_without_ext)
            if r['return'] > 0:
                return r

            base = r['meta']

            base_path_meta[path] = copy.deepcopy(base)

        for k in meta:
            v = meta[k]

            if k not in base:
                base[k] = v
            else:
                if isinstance(v, str):
                    # Only merge a few special keys and overwrite the rest
                    if k in ['tags', 'name']:
                        base[k] += meta[k]
                    else:
                        base[k] = meta[k]


elif isinstance(v,                 elif)                    for vv in v:
                        base[k].append(vv)
elif isinstance(v,                 elif )                    base[k].merge(v)

        meta = base

    return {'return': 0, 'meta':meta}


##########################################################################
def get_with_complex_key(meta, key):

    j = key.find('.')

    if j <0:
        return meta.get(key)

    key0 = key[:j]

    if key0 not in meta:
        return None

    return get_with_complex_key(meta[key0], key[j +1:])

##########################################################################

def get_with_complex_key_safe(meta, key):
    v = get_with_complex_key(meta, key)

    if v == None:
        v=''

    return v

##########################################################################

def prepare_table(i):

    import pandas as pd
    import numpy as np

    selection = i['selection']
    misc = i['misc_module']

    html = ''

    all_data = []

#    dimensions = [('input.model', 'MLPerf model'),
#                  ('input.implementation', 'MLPerf implementation'),
#                  ('input.framework', 'MLPerf framework')]

    dimensions = i.get('dimensions', [])

    dimension_values = {}
    dimension_keys = []

    if len(dimensions) == 0:
        keys = [('test', 'CM test', 400, 'leftAligned')]
    else:
        keys = [('test', 'CM test', 50, 'leftAligned')]

        for k in dimensions:
            key = k[0]

            keys.append((k[0], k[1], 100, 'leftAligned'))

            dimension_values[key] = []
            dimension_keys.append(key)

#        # assemble all values
#        for s in selection:
#            for k in dimensions:
#                key = k[0]
#
#                value = get_with_complex_key(selection, key)
#
#                if value!=None and value!='' and value not in dimension_values[key]:
#                    dimension_values.append(value)

        # If dimensions, sort by dimensions
        for d in list(reversed(dimension_keys)):
            selection = sorted(selection, key= lambda x: get_with_complex_key_safe(selection, d))


    keys += [
        ('functional', '<a href="https://github.com/mlcommons/ck/blob/master/docs/artifact-evaluation/submission.md">Functional</a>', 80, ''),
             ('reproduced', '<a href="https://github.com/mlcommons/ck/blob/master/docs/artifact-evaluation/submission.md">Reproduced</a>', 80, ''),
             ('notes', 'Notes', 200, 'lefAligned'),
        ]

    j = 0

    badges_url ={'functional':'https://cTuning.org/images/artifacts_evaluated_functional_v1_1_small.png',
                'reproduced': 'https://cTuning.org/images/results_reproduced_v1_1_small.png'}


    for s in selection:
        row = {}

        j += 1

        uid = s['uid']

        url = misc.make_url(uid, key='uid', action='howtorun', md=False)

        name = s.get('name', '')
        if name == '':
            name = uid

        if len(dimensions) == 0:
            row['test'] = '<a href="{}" target="_blank">{}</a>'.format(
                url, name)
        else:
            row['test'] = '<a href="{}" target="_blank">View</a>'.format(url)
            for k in dimensions:
                kk = k[0]

                v = get_with_complex_key_safe(s, kk)

                row[kk] = str(v)


        # Check ACM/IEEE functional badge
        x = ''
        if s.get('functional', False):
            x = '<center><a href="{}" target="_blank"><img src="{}" height="32"></a></center>'.format(
                url, badges_url['functional'])
        row['functional'] = x

        # Check ACM/IEEE reproduced badge
        x = ''
        if s.get('reproduced', False):
            x = '<center><a href="{}" target="_blank"><img src="{}" height="32"></a></center>'.format(
                url, badges_url['reproduced'])
        row['reproduced'] = x

        # Check misc notes
        row['notes'] = s.get('notes','')

        # Finish row
        all_data.append(row)

    # Visualize table
    pd_keys = [v[0] for v in keys]
    pd_key_names = [v[1] for v in keys]

    pd_all_data = []
    for row in sorted(all_data, key=lambda row: (row.get('x1', 0))):
        pd_row = []
        for k in pd_keys:
            pd_row.append(row.get(k))
        pd_all_data.append(pd_row)

    df = pd.DataFrame(pd_all_data, columns= pd_key_names)

    df.index += 1

    return {'return': 0, 'df':df}


##########################################################################
def gui(i):

    params = i['params']
    st = i['streamlit_module']
    misc = i['misc_module']
    meta = i['meta']
    gui_meta = meta['gui']
    skip_header = i.get('skip_title', False)

    end_html = ''

    if not skip_header:
        # Title
        title = gui_meta['title']

        st.title('[Collective Mind](https://github.com/mlcommons/ck)')

        st.markdown('### {}'.format(title))


    # Check if test uid is specified
    uid = ''
    x = params.get('uid', [''])
    if len(x)>0 and x[0]!='':
        uid = x[0].strip()

    bench_uid = ''
    x = params.get('bench_uid', [''])
    if len(x)>0 and x[0]!='':
        bench_uid = x[0].strip()

    compute_uid = ''
    x = params.get('compute_uid', [''])
    if len(x)>0 and x[0]!='':
        compute_uid = x[0].strip()


    ##############################################################
    # Check the first level of benchmarks
    ii = {'tags': 'benchmark,run', 'skip_files':True, 'prune':{}}

    if uid != '':
        ii['skip_files'] = False
        ii['prune']['uid'] = uid
    if bench_uid != '':
        ii['artifact'] = bench_uid
    if compute_uid != '':
        ii['prune']['key'] = 'supported_compute'
        ii['prune']['key_uid'] = compute_uid

    r = load_cfg(ii)
    if r['return']>0:
        return r

    lst = r['selection']

    if len(lst) ==0:
        st.markdown('Warning: no benchmarks found!')
        return {'return': 0}

    test_meta = {}

    bench_id = 0

    ##########################################################################
    if uid != '':
        if len(lst) ==0:
            st.markdown('CM test with UID "{}" not found!'.format(uid))
            return {'return': 0}
        elif len(lst) >1:
            st.markdown(
    'Warning: More than 1 CM test found with UID "{}" - ambiguity!'.format(uid))
            return {'return': 0}

        test_meta = lst[0]

        bench_id = 1
        compute_uid = test_meta['compute_uid']
        bench_supported_compute = [compute_uid]

    if uid == '':
        selection = sorted(lst, key= lambda v: v['name'])
        bench_selection = [{'name': ''}] + selection

        if bench_uid != '':
            bench_id_index = 1
        else:
            # Check if want to force some benchmark by default
            # 27c06c35bceb4059 == MLPerf inference v4.0

            bench_id_index = 0

            j = 0
            for b in bench_selection:
                if b.get('uid','') =='27c06c35bceb4059':
                    bench_id_index = j
                    break
                j += 1

        bench_id = st.selectbox('Select benchmark:',
                                range(len(bench_selection)),
                                format_func=lambda x: bench_selection[x]['name'],
                                index = bench_id_index,
                                key = 'bench')

        bench_supported_compute = []
        bench_meta = {}
        if bench_id >0:
            bench_meta = bench_selection[bench_id]
            bench_supported_compute = bench_meta.get('supported_compute', [])

            urls = bench_meta.get('urls', [])
            if len(urls) >0:
                x = '\n'
                for u in urls:
                    name = u['name']
                    url = u['url']

                    x += ' [ [{}]({}) ] '.format(name, url)
                x += '\n'

                st.markdown(x)

    ##########################################################################
    if True ==True:
        ##############################################################
        # Check compute

        ii = {'tags': 'benchmark,compute'}
        if bench_id >0:
            if compute_uid != '':
                x = [compute_uid]
            else:
                x = bench_supported_compute
                if len(x) == 0:
                    st.markdown('Warning: no supported compute selected!')
                    return {'return': 0}

            ii['prune'] = {'list':x}

        r = load_cfg(ii)
        if r['return']>0:
            return r

        selection = sorted(r['selection'], key= lambda v: v['name'])

        if len(selection) == 0:
            st.markdown('Warning: no supported compute found!')
            return {'return': 0}

        compute_selection = [{'name': ''}]
        if len(selection) >0:
            compute_selection += selection

        compute_id_index = 0 if compute_uid == '' else 1

        if uid == '':
            compute_id = st.selectbox('Select target hardware to benchmark:',
                                      range(len(compute_selection)),
                                      format_func=lambda x: compute_selection[x]['name'],
                                      index = compute_id_index,
                                      key = 'compute')

            compute = {}
            if compute_id >0:
                compute = compute_selection[compute_id]
                compute_uid = compute['uid']

        compute_meta = {}
        for c in compute_selection:
            if c.get('uid','') !='':
                compute_meta[c['uid']] = c

    ##########################################################################
    if uid == '':

        ##############################################################
        # Check tests
        ii = {'tags': 'benchmark,run'}

        if bench_id >0:
            bench_uid = bench_selection[bench_id]['uid']
            ii['artifact'] = bench_uid
        if compute_uid !='':
            ii['prune'] = {'key':'compute_uid', 'key_uid':compute_uid}

        r = load_cfg(ii)
        if r['return']>0:
            return r

        selection = sorted(r['selection'], key= lambda v: v['name'])

        # Check how many and prune
        if len(selection) == 0:
            st.markdown('No CM tests found')
            return {'return': 0}

        for s in selection:
            c_uid = s.get('compute_uid', '')
            if c_uid !='':
                c_tags = compute_meta[c_uid].get('tags', '')
                if c_tags !='':
                    s['all_tags'] += c_tags.split(',')

                s['compute_meta'] = compute_meta[c_uid]


        if len(selection) >1:
            # Update selection with compute tags
            test_tags = ''
            x = params.get('tags', [''])
            if len(x)>0 and x[0]!='':
                test_tags = x[0].strip()

            test_tags = st.text_input('Found {} CM tests. Prune them by tags:'.format(
                str(len(selection))), value=test_tags, key='test_tags').strip()

            if test_tags !='':
                test_tags_list = test_tags.replace(' ', ',').split(',')

                pruned_selection = []

                for s in selection:
                    all_tags = s['all_tags']

                    add = True

                    for t in test_tags_list:
                        if t not in all_tags:
                            add = False
                            break

                    if add:
                        pruned_selection.append(s)

                selection = pruned_selection

        test_selection = [{'name': ''}] + selection



        if len(selection) <200:
            # Creating compute selector
            test_id_index = 1 if len(selection) == 1 else 0

            test_id = st.selectbox('Select a test from {}:'.format(str(len(selection))),
                                   range(len(test_selection)),
                                   format_func=lambda x: test_selection[x]['name'],
                                   index = test_id_index,
                                   key = 'test')


            if test_id > 0:
                test_meta = test_selection[test_id]
            else:
                ###############################################################
                # View many (table)
                ii = {'selection': selection,
                      'misc_module': misc}

                # Check if dimensions in the bench
                dimensions = bench_meta.get('dimensions', [])
                if len(dimensions) >0:
                    viewer_selection = ['benchmark specific', 'universal']

                    viewer = st.selectbox('Viewer:', viewer_selection, key= 'viewer')

                    if viewer == 'benchmark specific':
                        ii['dimensions'] = dimensions

                else:
                    st.markdown('---')

                r = prepare_table(ii)
                if r['return']>0:
                    return r

                df = r['df']

                html = df.to_html(escape=False, justify='left')
                st.write(html, unsafe_allow_html= True)

#                st.dataframe(df, unsafe_allow_html = True)


    ##############################################################
    # Show individual test
    if len(test_meta) >0:
        if uid != '':
            c_uid = test_meta.get('compute_uid', '')
            if c_uid !='':
                c_tags = compute_meta[c_uid].get('tags', '')
                if c_tags !='':
                    test_meta['all_tags'] += c_tags.split(',')

                test_meta['compute_meta'] = compute_meta[c_uid]

        if uid == '':
            st.markdown('---')

            uid = test_meta['uid']

        # First, check if there is a README
        test_path = test_meta['full_path']

        test_md = test_meta['full_path'][:-5] +'.md'
        if os.path.isfile(test_md):

            r = cmind.utils.load_txt(test_md)
            if r['return']>0:
                return r

            s = r['string']

            st.markdown(s)

        # Next print some info (for now JSON)
        import json
        x = """
---
**CM test dictionary:**
```json
{}
```
            """.format(json.dumps(test_meta, indent=2))
        st.markdown(x)


        # Create self link
        # This misc module is in CM "gui" script
        x1 = misc.make_url(uid, key='uid', action='howtorun', md=False)
        end_html = '<center><small><i><a href="{}">Self link</a></i></small></center>'.format(x1)

    return {'return': 0, 'end_html': end_html}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']
    automation = i['automation']
    meta = i['meta']
    os_info = i['os_info']

    env['CM_DATASET_IMAGENET_VAL_REQUIRE_DAE'] = 'no'

    full = env.get('CM_IMAGENET_FULL', '').strip() == 'yes'

    path = env.get(
        'CM_INPUT',
        env.get(
            'IMAGENET_PATH',
            env.get(
                'CM_DATASET_IMAGENET_PATH',
                ''))).strip()

    if path == '':
        if full:

            if env.get('CM_DATASET_IMAGENET_VAL_TORRENT_PATH'):
                path = env['CM_DATASET_IMAGENET_VAL_TORRENT_PATH']
                env['CM_DAE_EXTRA_TAGS'] = "_torrent"
                env['CM_DAE_TORRENT_PATH'] = path
                env['CM_DATASET_IMAGENET_VAL_REQUIRE_DAE'] = 'yes'
                return {'return': 0}

            else:
                env['CM_DAE_URL'] = 'https://image-net.org/data/ILSVRC/2012/ILSVRC2012_img_val.tar'
                env['CM_DAE_FILENAME'] = 'ILSVRC2012_img_val.tar'
                env['CM_DATASET_IMAGENET_VAL_REQUIRE_DAE'] = 'yes'

                return {'return': 0}
                # return {'return':1, 'error':'Please rerun the last CM command
                # with --env.IMAGENET_PATH={path the folder containing full
                # ImageNet images} or envoke cm run script "get val dataset
                # imagenet" --input={path to the folder containing ImageNet
                # images}'}

        else:
            env['CM_DATASET_IMAGENET_VAL_REQUIRE_DAE'] = 'yes'

    elif not os.path.isdir(path):
        if path.endswith(".tar"):
            env['CM_EXTRACT_FILEPATH'] = path
            env['CM_DAE_ONLY_EXTRACT'] = 'yes'
            return {'return': 0}
        else:
            return {'return': 1,
                    'error': 'Path {} doesn\'t exist'.format(path)}
    else:
        env['CM_EXTRACT_EXTRACTED_PATH'] = path

    return {'return': 0}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']
    path = env['CM_EXTRACT_EXTRACTED_PATH']
    path1 = os.path.join(path, 'imagenet-2012-val')
    if os.path.isdir(path1):
        path = path1

    path_image = os.path.join(path, 'ILSVRC2012_val_00000001.JPEG')

    if not os.path.isfile(path_image):
        return {'return': 1,
                'error': 'ImageNet file {} not found'.format(path_image)}

    files = os.listdir(path)
    if len(files) < int(env.get('CM_DATASET_SIZE', 0)):
        return {'return': 1, 'error': 'Only {} files found in {}. {} expected'.format(
            len(files), path, env.get('CM_DATASET_SIZE'))}

    env['CM_DATASET_PATH'] = path
    env['CM_DATASET_IMAGENET_PATH'] = path
    env['CM_DATASET_IMAGENET_VAL_PATH'] = path

    env['CM_GET_DEPENDENT_CACHED_PATH'] = path

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

import cmind as cm
from cmind import utils

import os
import subprocess
import json

file_summary_json = 'mlperf-inference-summary.json'
file_result = 'cm-result.json'

fix_benchmark_names = {'anomaly_detection': 'ad',
                       'image_classification': 'ic',
                       'keyword_spotting': 'kws',
                       'visual_wake_words': 'vww'}


def preprocess(i):

    env = i['env']

    cur_dir = os.getcwd()

    # Query cache for results dirs
    r = cm.access({'action': 'find',
                   'automation': 'cache,541d6f712a6b464e',
                   'tags': 'get,repo,mlperf-tiny-results'})
    if r['return'] > 0:
        return r

    lst = r['list']

    for c in lst:
        path = os.path.join(c.path, 'repo')

        if os.path.isdir(path):
            meta = c.meta

            tags = meta['tags']

            version = ''
            for t in tags:
                if t.startswith('version-'):
                    version = 'v' + t[8:]
                    break

            r = convert_repo_to_experiment(path, version, env)
            if r['return'] > 0:
                return r

    print('')

    return {'return': 0}


def convert_repo_to_experiment(path, version, env):
    print('')
    print('Processing MLPerf repo from CM cache path: {}'.format(path))
    print('* Version: {}'.format(version))

    cur_dir = os.getcwd()

    # Get Git URL
    os.chdir(path)

    burl = subprocess.check_output(
        ['git', 'config', '--get', 'remote.origin.url'])
    url = burl.decode('UTF-8').strip()

    print('* Git URL: {}'.format(url))

    # Create virtual experiment entries
    experiments = {}

    for division in ['closed', 'open']:
        p1 = os.path.join(path, division)
        if os.path.isdir(p1):
            print('  * Processing division: {}'.format(division))

            companies = os.listdir(p1)

            for company in companies:
                p2 = os.path.join(p1, company)
                if os.path.isdir(p2):
                    print('    * Processing company: {}'.format(company))

                    presults = os.path.join(p2, 'results')
                    psystems = os.path.join(p2, 'systems')
                    pcode = os.path.join(p2, 'code')

                    if os.path.isdir(presults) and os.path.isdir(
                            psystems) and os.path.isdir(pcode):
                        #  Exception for OctoML
                        presults2 = [presults]

                        if company == 'OctoML' and version == 'v1.0':
                            presults2 = []

                            p3 = os.listdir(presults)
                            for p3x in p3:
                                p3y = os.path.join(presults, p3x)
                                if os.path.isdir(p3y):
                                    presults2.append(p3y)

                        for presult in presults2:
                            systems = os.listdir(presult)
                            for system in systems:
                                psystem = os.path.join(presult, system)
                                if os.path.isdir(psystem):
                                    print(
                                        '      * Processing result for system: {}'.format(system))

                                    # Check system file
                                    psystem_desc = os.path.join(
                                        psystems, system + '.json')
                                    psystem_dict = {}

                                    print(
                                        '                                File: {}'.format(psystem_desc))

                                    # Check exceptions
                                    if version == 'v1.0':
                                        if company == 'OctoML':
                                            x = os.path.basename(presult)
                                            psystem_desc = os.path.join(
                                                psystems,
                                                'system_description_' +
                                                system.replace(
                                                    '-',
                                                    '') +
                                                '_' +
                                                x +
                                                '.json')
                                        elif company == 'STMicroelectronics':
                                            psystem_desc = os.path.join(
                                                psystems, system, system + '_system_description.json')
                                            if not os.path.isfile(
                                                    psystem_desc):
                                                psystem_desc = os.path.join(
                                                    psystems, system, system.replace(
                                                        '-', '_') + '_system_description.json')
                                        elif company == 'syntiant':
                                            psystem_desc = os.path.join(
                                                psystems, system, system + '.json')
                                        elif company == 'hls4ml':
                                            psystem_desc = os.path.join(
                                                psystems, 'system_description_pynq.json')
                                    elif version == 'v0.7':
                                        if company == 'renesas':
                                            psystem_desc = os.path.join(
                                                psystems, system + '_System_Description.json')
                                        elif company == 'STMicroelectronics':
                                            psystem_desc = os.path.join(
                                                psystems, system, system + '_system_description.json')
                                            if not os.path.isfile(
                                                    psystem_desc):
                                                psystem_desc = os.path.join(
                                                    psystems, system, system.replace(
                                                        '-', '_') + '_system_description.json')
                                        elif company == 'syntiant':
                                            psystem_desc = os.path.join(
                                                psystems, system, system + '.json')
                                        elif company == 'hls4ml-finn':
                                            psystem_desc = os.path.join(
                                                psystems, 'system_description_' + system[:4] + '.json')

                                    if os.path.isfile(psystem_desc):
                                        x = ''
                                        if version == 'v1.0':
                                            if company == 'OctoML':
                                                x = '}\n\t"'
                                            elif company == 'syntiant':
                                                x = '"\n\t"'
                                            elif company == 'hls4ml':
                                                x = 'dummy'
                                        elif version == 'v0.7':
                                            if company == 'syntiant':
                                                x = '"\n\t"'

                                        if x != '':
                                            r = utils.load_txt(psystem_desc)
                                            if r['return'] > 0:
                                                return r

                                            s = r['string']

                                            j = s.find(x)
                                            if j >= 0:
                                                s = s[:j + 1] + ',' + s[j + 1:]

                                            if s.endswith(',\n'):
                                                s = s[:-2] + '}'

                                            psystem_dict = json.loads(s)

                                        else:
                                            r = utils.load_json(psystem_desc)
                                            if r['return'] > 0:
                                                return r
                                            psystem_dict = r['meta']

                                    else:
                                        print(
                                            '           * Warning: system description not found in {}'.format(psystem_desc))
                                        input(
                                            '             Press <Enter> to continue')

                                    for benchmark in os.listdir(psystem):
                                        pbenchmark = os.path.join(
                                            psystem, benchmark)
                                        if os.path.isdir(pbenchmark):
                                            print(
                                                '         * Processing benchmark: {}'.format(benchmark))

                                            models = ['']

                                            # May have retrained models
                                            pperf = os.path.join(
                                                pbenchmark, 'performance', 'results.txt')
                                            if not os.path.isfile(pperf):
                                                pperf = os.path.join(
                                                    pbenchmark, 'performance', 'performance_results.txt')

                                            if not os.path.isfile(pperf):
                                                # likely models
                                                models = []

                                                for model in os.listdir(
                                                        pbenchmark):
                                                    pmodel = os.path.join(
                                                        pbenchmark, model)
                                                    if os.path.isdir(pmodel):
                                                        models.append(model)

                                            for model in models:

                                                results = {}

                                                if model != '':
                                                    print(
                                                        '           * Processing model: {}'.format(model))
                                                    pbenchmark = os.path.join(
                                                        psystem, benchmark, model)

                                                perf_file_type = 0
                                                pperf = os.path.join(
                                                    pbenchmark, 'performance', 'results.txt')
                                                if not os.path.isfile(pperf):
                                                    pperf = os.path.join(
                                                        pbenchmark, 'performance', 'performance_results.txt')
                                                    perf_file_type = 1  # outdated/weird

                                                paccuracy = os.path.join(
                                                    pbenchmark, 'accuracy', 'results.txt')
                                                if not os.path.isfile(
                                                        paccuracy):
                                                    paccuracy = os.path.join(
                                                        pbenchmark, 'accuracy', 'accuracy_results.txt')

                                                penergy = os.path.join(
                                                    pbenchmark, 'energy', 'results.txt')

                                                if os.path.isfile(
                                                        pperf) and os.path.isfile(paccuracy):
                                                    r = utils.load_txt(pperf)
                                                    if r['return'] > 0:
                                                        return r

                                                    s = r['string']

                                                    median_throughput = 0

                                                    x1 = 'Median throughput is ' if perf_file_type == 0 else 'Throughput   :'
                                                    x2 = 21 if perf_file_type == 0 else 18

                                                    j = s.find(x1)
                                                    if j >= 0:
                                                        j1 = s.find(
                                                            ' inf./sec.', j)
                                                        if j1 >= 0:
                                                            median_throughput = float(
                                                                s[j + x2:j1].strip())
                                                            results['median_throughput'] = median_throughput
                                                            results['median_throughput_metric'] = 'inf./sec.'
                                                            results['Result'] = median_throughput
                                                            results['_Result'] = median_throughput

                                                    if median_throughput == 0:
                                                        print(
                                                            '           * Warning: median_throughput was not detected in {}'.format(pperf))
                                                        input(
                                                            '             Press <Enter> to continue')

                                                    r = utils.load_txt(
                                                        paccuracy, split=True)
                                                    if r['return'] > 0:
                                                        return r

                                                    lines = r['list']

                                                    found = False

                                                    for line in lines:
                                                        j = line.find(
                                                            'ulp-mlperf: ')
                                                        if j >= 0:
                                                            j1 = line.find(
                                                                ':', j + 12)
                                                            if j1 >= 0:
                                                                accuracy_key = 'accuracy_' + \
                                                                    line[j + 12:j1]
                                                                value = line[j1 + 2:]

                                                                if value.endswith(
                                                                        '%'):
                                                                    value = value[:-1]
                                                                    results[accuracy_key +
                                                                            '_metric'] = '%'

                                                                value = float(
                                                                    value)

                                                                results[accuracy_key] = value

                                                                if not found:
                                                                    # first
                                                                    # value
                                                                    results['Accuracy'] = value
                                                                    results['_Accuracy'] = value

                                                                found = True

                                                    if not found:
                                                        print(
                                                            '           * Warning: accuracy not found in the file {}'.format(paccuracy))
                                                        input(
                                                            '             Press <Enter> to continue')

                                                else:
                                                    print(
                                                        '           * Warning: performance or accuracy files are not present in this submission')
                                                    input(
                                                        '             Press <Enter> to continue')

                                                if os.path.isfile(penergy):
                                                    r = utils.load_txt(penergy)
                                                    if r['return'] > 0:
                                                        return r

                                                    s = r['string']

                                                    median_throughput = 0

                                                    j = s.find(
                                                        'Median throughput is ')
                                                    if j >= 0:
                                                        j1 = s.find(
                                                            ' inf./sec.', j)
                                                        if j1 >= 0:
                                                            median_throughput = float(
                                                                s[j + 21:j1])

                                                            results['median_energy_median_throughput'] = median_throughput
                                                            results['median_energy_median_throughput_metric'] = 'inf./sec.'

                                                    if median_throughput == 0:
                                                        print(
                                                            '           * Warning: median_throughput was not detected in {}'.format(penergy))
                                                        input(
                                                            '             Press <Enter> to continue')
                                                    else:
                                                        median_energy_cost = 0

                                                        j = s.find(
                                                            'Median energy cost is ')
                                                        if j >= 0:
                                                            j1 = s.find(
                                                                ' uJ/inf.', j)
                                                            if j1 >= 0:
                                                                median_energy_cost = float(
                                                                    s[j + 22:j1])

                                                                results['median_energy_cost'] = median_energy_cost
                                                                results['median_energy_cost_metric'] = 'uj/inf.'

                                                        if median_energy_cost == 0:
                                                            print(
                                                                '           * Warning: median_energy_cost was not detected in {}'.format(penergy))
                                                            input(
                                                                '             Press <Enter> to continue')

                                                print(
                                                    '           * Results dict: {}'.format(results))

                                                # Finalizing keys
                                                results.update(psystem_dict)

                                                xbenchmark = benchmark if benchmark not in fix_benchmark_names else fix_benchmark_names[
                                                    benchmark]

                                                results['git_url'] = url + \
                                                    '/tree/master/' + division + '/' + company

                                                results['version'] = version
                                                results['__version'] = version
                                                results['Organization'] = company
                                                results['__Organization'] = company
                                                results['Division'] = division
                                                results['Benchmark'] = xbenchmark
                                                results['__System'] = system

                                                if model != '':
                                                    results['Model'] = model
                                                    results['__Model'] = model

                                                # Prepare experiment name
                                                cm_name = 'mlperf-tiny--{}--' + division + '--' + xbenchmark
                                                print(
                                                    '           * CM experiment name: {}'.format(cm_name))

                                                name_all = cm_name.format(
                                                    'all')
                                                name_ver = cm_name.format(
                                                    version)

                                                for name in [
                                                        name_all, name_ver]:
                                                    if name not in experiments:
                                                        experiments[name] = []
                                                    experiments[name].append(
                                                        results)

                    else:
                        print(
                            '      * Warning: some directories are not present in this submission')
                        input('        Press <Enter> to continue')

    os.chdir(cur_dir)

    r = utils.save_json(file_summary_json, experiments)
    if r['return'] > 0:
        return r

    env_target_repo = env.get('CM_IMPORT_TINYMLPERF_TARGET_REPO', '').strip()
    target_repo = '' if env_target_repo == '' else env_target_repo + ':'

    # Checking experiment
    print('')
    for name in experiments:
        print('    Preparing experiment artifact "{}"'.format(name))

        tags = name.split('--')
        if 'mlperf' not in tags:
            tags.insert(0, 'mlperf')

        # Checking if experiment already exists
        r = cm.access({'action': 'find',
                       'automation': 'experiment,a0a2d123ef064bcb',
                       'artifact': target_repo + name})
        if r['return'] > 0:
            return r

        lst = r['list']

        if len(lst) == 0:
            r = cm.access({'action': 'add',
                           'automation': 'experiment,a0a2d123ef064bcb',
                           'artifact': target_repo + name,
                           'tags': tags})
            if r['return'] > 0:
                return r

            path = r['path']
        else:
            path = lst[0].path

        results = experiments[name]

        # Check if already date directory
        dirs = os.listdir(path)

        path2 = ''
        for d in dirs:
            dd = os.path.join(path, d)
            if os.path.isdir(dd):
                path2 = dd
                break

        if path2 == '':

            r = utils.get_current_date_time({})
            if r['return'] > 0:
                return r

            date_time = r['iso_datetime'].replace(':', '-').replace('T', '.')

            path2 = os.path.join(path, date_time)

            os.makedirs(path2)

        # Check if cm-result.json
        fresult = os.path.join(path2, file_result)

        if os.path.isfile(fresult):
            r = utils.load_json(fresult)
            if r['return'] > 0:
                return r

            existing_results = r['meta']

            # Need to check which ones to add
            for result in existing_results:
                found = False

                # New results
                for result2 in results:
                    matched = True

                    # Need to iterate over keys in the new results since old
                    # results can have more keys (derivates, etc)
                    for k in result2:
                        if k != 'uid':
                            if k not in result or result2[k] != result[k]:
                                matched = False
                                break

                    if matched:
                        found = True
                        break

                if not found:
                    results.append(result)

        # Check extra keys
        final_results = []
        for result in results:
            # Generate UID
            if 'uid' not in result:
                r = utils.gen_uid()
                if r['return'] > 0:
                    return r

                result['uid'] = r['uid']

        # Write results
        r = utils.save_json(fresult, results)
        if r['return'] > 0:
            return r

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    # CM script internal variables
    variation_tags = i.get('variation_tags', [])
    automation = i['automation']
    env = i['env']
    meta = i['meta']
    quiet = (env.get('CM_QUIET', False) == 'yes')

    # Check if path is there to detect existing data set
    detected = False
    path = env.get('CM_TMP_PATH', '')
    if path != '':
        if not os.path.isdir(path):
            return {'return': 1,
                    'error': 'path to dataset "{}" doesn\'t exist'.format(path)}

        # Check which dataset
        p = os.path.join(path, 'annotations')
        if os.path.isdir(p):
            for d in [('val2017', 'val', '2017'),
                      ('train2017', 'train', '2017')]:
                p = os.path.join(path, d[0])

                if os.path.isdir(p):
                    tp = d[1]
                    ver = d[2]
                    detected = True
                    break

        if not detected:
            return {
                'return': 1, 'error': 'COCO dataset is not detected in "{}"'.format(path)}

        print('')
        print('Detected COCO dataset {} {}'.format(tp, ver))

        env['CM_DATASET_COCO_DETECTED'] = 'yes'
        env['CM_DATASET_COCO_PATH'] = path
    else:
        ver = env['CM_DATASET_COCO_VERSION']
        tp = env['CM_DATASET_COCO_TYPE']

    # Prepare URL
    size = env.get('CM_DATASET_COCO_SIZE', '')
    if size == 'small' and tp == 'val' and ver == '2017':
        # We prepared a small version with 50 images for val 2017

        filename_data = 'val2017_small.zip'
        filename_annotation = 'annotations_val2017_small.zip'

        url_data_full = 'https://www.dropbox.com/scl/fi/whokyb7b7hyjqqotruyqb/{}?rlkey=hhgt4xtir91ej0nro6h69l22s&dl=0'.format(
            filename_data)
        url_ann_full = 'https://www.dropbox.com/scl/fi/bu41y62v9zqhee8w7q6z3/{}?rlkey=seqtgozldkc0ztu76kbd47p5w&dl=0'.format(
            filename_annotation)

    else:
        url_data = env['CM_DATASET_COCO_URL_DATA']
        url_ann = env['CM_DATASET_COCO_URL_ANNOTATIONS']

        filename_data = tp + ver + '.zip'
        filename_annotation = 'annotations_trainval' + ver + '.zip'

        url_data_full = url_data + '/' + filename_data
        url_ann_full = url_ann + '/' + filename_annotation

    # Add extra tags with type and version to "download-and-extract" deps to be able to reuse them
    # Add "from" and "to" to "download-and-extract" deps
    download_extra_cache_tags = 'dataset,coco,data,' + tp + ',' + ver

    dae_input_data = {
        'extra_cache_tags': download_extra_cache_tags
    }
    dae_input_annotation = {
        'extra_cache_tags': download_extra_cache_tags
    }

    path_from = env.get('CM_FROM', '')
    if path_from != '':
        path_from_data = os.path.join(path_from, filename_data)
        if not os.path.isfile(path_from_data):
            return {'return': 1,
                    'error': 'File {} not found'.format(path_from_data)}
        dae_input_data['local_path'] = path_from_data

        path_from_annotation = os.path.join(path_from, filename_annotation)
        if not os.path.isfile(path_from_annotation):
            return {'return': 1, 'error': 'File {} not found'.format(
                path_from_annotation)}
        dae_input_annotation['local_path'] = path_from_annotation

    path_to = env.get('CM_TO', '')
    if path_to != '':
        dae_input_data['extract_path'] = path_to
        dae_input_annotation['extract_path'] = path_to

    path_store = env.get('CM_STORE', '')
    if path_store != '':
        dae_input_data['download_path'] = path_store
        dae_input_data['tags'] = '_keep'
        dae_input_annotation['download_path'] = path_store
        dae_input_annotation['tags'] = '_keep'

    r = automation.update_deps({'deps': meta['prehook_deps'],
                                'update_deps': {
        '746e5dad5e784ad6': dae_input_data,
        'edb6cd092ff64171': dae_input_annotation
    }
    })
    if r['return'] > 0:
        return r

    # Prepare environment variables
    env['CM_DATASET_COCO_VERSION'] = ver
    env['CM_DATASET_COCO_TYPE'] = tp
    env['CM_DATASET_COCO_TYPE_AND_VERSION'] = tp + ver
    env['CM_DATASET_COCO_URL_DATA_FULL'] = url_data_full
    env['CM_DATASET_COCO_URL_ANNOTATIONS_FULL'] = url_ann_full

    # Check MD5SUM
    md5sum_data = ''
    md5sum_ann = ''

    if ver == '2017':
        if tp == 'val':
            if size == 'small':
                md5sum_data = '16fab985a33afa66beeb987f68c2023c'
                md5sum_ann = '78c0cfd9fc32c825d4ae693fd0d91407'
            else:
                md5sum_data = '442b8da7639aecaf257c1dceb8ba8c80'
                md5sum_ann = 'f4bbac642086de4f52a3fdda2de5fa2c'

    if md5sum_data != '':
        env['CM_DATASET_COCO_MD5SUM_DATA'] = md5sum_data
    if md5sum_ann != '':
        env['CM_DATASET_COCO_MD5SUM_ANN'] = md5sum_ann

    if not detected:
        print('')
        print('URL for data: {}'.format(url_data_full))
        print('URL for annotations: {}'.format(url_ann_full))

    # Add version and type to tags
    extra_cache_tags = []
    for tag in [ver, tp]:
        if tag not in variation_tags:
            extra_cache_tags.append(tag)

    return {'return': 0, 'add_extra_cache_tags': extra_cache_tags}


def postprocess(i):

    os_info = i['os_info']

    env = i['env']

    tp_ver = env['CM_DATASET_COCO_TYPE_AND_VERSION']

    path_to = env.get('CM_TO', '')

    # Check if detected or downloaded
    if env.get('CM_DATASET_COCO_DETECTED',
               '').lower() == 'yes' or path_to != '':
        path_all = env['CM_DATASET_COCO_PATH'] if path_to == '' else path_to

        env['CM_DATASET_COCO_DATA_PATH'] = os.path.join(path_all, tp_ver)
        env['CM_DATASET_COCO_ANNOTATIONS_PATH'] = os.path.join(
            path_all, 'annotations')
    else:
        path_all = os.getcwd()

        # Moving 2 directories to 1 place

        path_data = env['CM_DATASET_COCO_DATA_PATH']
        path_ann = env['CM_DATASET_COCO_ANNOTATIONS_PATH']

        print('')
        print(path_all)
        print('')

        path_data_full = os.path.join(path_data, tp_ver)
        path_ann_full = os.path.join(path_ann, 'annotations')

        if os_info['platform'] == 'windows':
            # Moving to this directory since can't make symbolic links
            command1 = '  move /y ' + path_data_full + ' ' + tp_ver
            command2 = '  move /y ' + path_ann_full + ' annotations'

            env['CM_DATASET_COCO_DATA_PATH'] = os.path.join(path_all, tp_ver)
            env['CM_DATASET_COCO_ANNOTATIONS_PATH'] = os.path.join(
                path_all, 'annotations')
        else:
            # Make soft links from data and annotations into 1 directory
            # (standard way for COCO)

            command1 = '  ln -s ' + path_data_full + ' ' + tp_ver
            command2 = '  ln -s ' + path_ann_full + ' annotations'

        for command in [command1, command2]:
            print(command)
            os.system(command)

    env['CM_DATASET_COCO_PATH'] = path_all
    env['CM_DATASET_PATH'] = path_all
    env['CM_DATASET_PATH_ROOT'] = path_all

    return {'return': 0}



# Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import torch
import onnx
import argparse
import json
import re

import onnx_graphsurgeon as gs
import numpy as np
import os


# in_onnx = "/work/code/retinanet/tensorrt/onnx_retina/ref_fpn_transreshapeconcat.onnx"
in_onnx = os.environ.get(
    "CM_ML_MODEL_DYN_BATCHSIZE_PATH",
    "build/models/retinanet-resnext50-32x4d/new/retinanet_resnext50_32x4d_fpn.opset11.dyn_bs.800x800.onnx")
out_onnx = os.environ.get(
    "CM_NVIDIA_MODEL_PATCHED_PATH",
    "/work/code/retinanet/tensorrt/onnx_generator/test_fpn_efficientnms_concatall.onnx")
# Anchor at [1, 1]
anchor_xywh_1x1_npy = os.environ.get(
    "CM_ML_MODEL_ANCHOR_PATH",
    "/work/code/retinanet/tensorrt/onnx_generator/retinanet_anchor_xywh_1x1.npy")

graph = gs.import_onnx(onnx.load(in_onnx))

op = 'EfficientNMS_TRT'
node_name = 'efficientNMS'

# (PluginField("score_threshold", nullptr, PluginFieldType::kFLOAT32, 1));
# (PluginField("iou_threshold", nullptr, PluginFieldType::kFLOAT32, 1));
# (PluginField("max_output_boxes", nullptr, PluginFieldType::kINT32, 1));
# (PluginField("background_class", nullptr, PluginFieldType::kINT32, 1));
# (PluginField("score_activation", nullptr, PluginFieldType::kINT32, 1));
# (PluginField("box_coding", nullptr, PluginFieldType::kINT32, 1));

node_attrs = {
    "background_class": -1,
    "score_threshold": 0.05,
    "iou_threshold": 0.5,
    "max_output_boxes": 1000,
    "score_activation": True,
    "box_coding": 1,
}
attrs = {
    "plugin_version": "1",
    "plugin_namespace": "",
}
attrs.update(node_attrs)

anchors = np.load(anchor_xywh_1x1_npy)
print(f"anchors shape: {anchors.shape}, top 4: {anchors[0, :]}")
anchors = np.expand_dims(anchors, axis=0)
print(f"anchors shape: {anchors.shape}")

anchor_tensor = gs.Constant(name="anchor", values=anchors)

tensors = graph.tensors()

# Add EfficientNMS layer
# output tensors
num_detections = gs.Variable(name="num_detections",
                             dtype=np.int32,
                             shape=["batch_size", 1])
detection_boxes = gs.Variable(name="detection_boxes",
                              dtype=np.float32,
                              shape=["batch_size", 1000, 4])
detection_scores = gs.Variable(name="detection_scores",
                               dtype=np.float32,
                               shape=["batch_size", 1000])
detection_classes = gs.Variable(name="detection_classes",
                                dtype=np.int32,
                                shape=["batch_size", 1000])

nms_inputs = [tensors["bbox_regression"], tensors["cls_logits"], anchor_tensor]
nms_outputs = [
    num_detections,
    detection_boxes,
    detection_scores,
    detection_classes]

graph.layer(op="EfficientNMS_TRT",
            name="EfficientNMS",
            inputs=nms_inputs,
            outputs=nms_outputs,
            attrs=attrs)

# Add Concat plugin to concat all 4 tensors
concat_final_output = gs.Variable(name="concat_final_output",
                                  dtype=np.float32,
                                  shape=["batch_size", 7001])
attrs = {
    "plugin_version": "1",
    "plugin_namespace": "",
}
graph.layer(op="RetinanetConcatNmsOutputsPlugin",
            name="RetinanetConcatNmsOutputsPlugin",
            inputs=[
                num_detections,
                detection_boxes,
                detection_scores,
                detection_classes],
            outputs=[concat_final_output],
            attrs=attrs)

graph.outputs = [concat_final_output]

graph.cleanup().toposort()

onnx.save_model(gs.export_onnx(graph), out_onnx)



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import shutil


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}
    env = i['env']

    if env.get('CM_MLPERF_SKIP_RUN', '') == "yes":
        return {'return': 0}

    if 'CM_MODEL' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the model to run'}
    if 'CM_MLPERF_BACKEND' not in env:
        return {'return': 1,
                'error': 'Please select a variation specifying the backend'}
    if 'CM_MLPERF_DEVICE' not in env:
        return {
            'return': 1, 'error': 'Please select a variation specifying the device to run on'}

    kilt_root = env['CM_KILT_CHECKOUT_PATH']

    print(f"Harness Root: {kilt_root}")

    source_files = []
    env['CM_SOURCE_FOLDER_PATH'] = env['CM_KILT_CHECKOUT_PATH']

    env['kilt_model_root'] = env.get('CM_ML_MODEL_FILE_WITH_PATH')

    if env.get('CM_MLPERF_LOADGEN_BATCH_SIZE', '') != '':
        env['kilt_model_batch_size'] = env['CM_MLPERF_LOADGEN_BATCH_SIZE']

    if env.get('CM_QAIC_DEVICES', '') != '':
        env['kilt_device_ids'] = env['CM_QAIC_DEVICES']

    if '+ CXXFLAGS' not in env:
        env['+ CXXFLAGS'] = []

    if '+CPLUS_INCLUDE_PATH' not in env:
        env['+CPLUS_INCLUDE_PATH'] = []

    if env['CM_MLPERF_DEVICE'] == "qaic":
        env['kilt_model_root'] = os.path.dirname(
            env['CM_QAIC_MODEL_COMPILED_BINARY_WITH_PATH'])

    if env.get('CM_MODEL') == "resnet50":
        env['dataset_imagenet_preprocessed_subset_fof'] = env['CM_DATASET_PREPROCESSED_IMAGENAMES_LIST']
        env['dataset_imagenet_preprocessed_dir'] = env['CM_DATASET_PREPROCESSED_PATH']

    elif "bert" in env.get('CM_MODEL'):
        env['dataset_squad_tokenized_max_seq_length'] = env['CM_DATASET_SQUAD_TOKENIZED_MAX_SEQ_LENGTH']
        env['dataset_squad_tokenized_root'] = env['CM_DATASET_SQUAD_TOKENIZED_ROOT']
        env['dataset_squad_tokenized_input_ids'] = os.path.basename(
            env['CM_DATASET_SQUAD_TOKENIZED_INPUT_IDS'])
        env['dataset_squad_tokenized_input_mask'] = os.path.basename(
            env['CM_DATASET_SQUAD_TOKENIZED_INPUT_MASK'])
        env['dataset_squad_tokenized_segment_ids'] = os.path.basename(
            env['CM_DATASET_SQUAD_TOKENIZED_SEGMENT_IDS'])

    elif "retinanet" in env.get('CM_MODEL'):
        env['kilt_prior_bin_path'] = os.path.join(
            kilt_root, "plugins", "nms-abp", "data")
        env['kilt_object_detection_preprocessed_subset_fof'] = os.path.basename(
            env['CM_DATASET_PREPROCESSED_IMAGENAMES_LIST'])
        env['kilt_object_detection_preprocessed_dir'] = env['CM_DATASET_PREPROCESSED_PATH']
        env['+ CXXFLAGS'].append("-DMODEL_RX50")
        env['+ CXXFLAGS'].append("-DSDK_1_11_X")

        loc_offset = env.get('CM_QAIC_MODEL_RETINANET_LOC_OFFSET')
        if loc_offset:
            env['+ CXXFLAGS'].append("-DMODEL_RX50")

        keys = ['LOC_OFFSET', 'LOC_SCALE', 'CONF_OFFSET', 'CONF_SCALE']

        if env.get('CM_RETINANET_USE_MULTIPLE_SCALES_OFFSETS', '') == 'yes':
            env['+ CXXFLAGS'].append("-DUSE_MULTIPLE_SCALES_OFFSETS=1")
            for j in range(0, 4):
                keys.append(f'LOC_OFFSET{j}')
                keys.append(f'LOC_SCALE{j}')
                keys.append(f'CONF_OFFSET{j}')
                keys.append(f'CONF_SCALE{j}')

        for key in keys:
            value = env.get('CM_QAIC_MODEL_RETINANET_' + key, '')
            if value != '':
                env['+ CXXFLAGS'].append(f" -D{key}_={value} ")

    if env.get('CM_BENCHMARK', '') == 'NETWORK_BERT_SERVER':
        source_files.append(
            os.path.join(
                kilt_root,
                "benchmarks",
                "network",
                "bert",
                "server",
                "pack.cpp"))
        source_files.append(
            os.path.join(
                kilt_root,
                "benchmarks",
                "network",
                "bert",
                "server",
                "server.cpp"))
        env['+ CXXFLAGS'].append("-DNETWORK_DIVISION=1")
    elif env.get('CM_BENCHMARK', '') == 'NETWORK_BERT_CLIENT':
        # source_files.append(os.path.join(kilt_root, "benchmarks", "network", "bert", "client", "pack.cpp"))
        # env['+CPLUS_INCLUDE_PATH'].append(kilt_root)
        # source_files.append(os.path.join(kilt_root, "benchmarks", "network", "bert", "client", "client.cpp"))
        env['+ CXXFLAGS'].append("-DNETWORK_DIVISION")
    elif env.get('CM_BENCHMARK', '') == 'STANDALONE_BERT':
        source_files.append(
            os.path.join(
                kilt_root,
                "benchmarks",
                "standalone",
                "bert",
                "pack.cpp"))

    script_path = i['run_script_input']['path']
    if env['CM_MODEL'] == "retinanet":
        env['CM_DATASET_LIST'] = env['CM_DATASET_ANNOTATIONS_FILE_PATH']

    for file in os.listdir(env['CM_SOURCE_FOLDER_PATH']):
        if file.endswith(".c") or file.endswith(".cpp"):
            source_files.append(file)

    if 'SERVER' not in env.get('CM_BENCHMARK', ''):
        source_files.append(
            os.path.join(
                kilt_root,
                "benchmarks",
                "harness",
                "harness.cpp"))

    # source_files.append(env['CM_QAIC_API_SRC_FILE'])

    env['+CPLUS_INCLUDE_PATH'].append(kilt_root)
    env['+C_INCLUDE_PATH'].append(kilt_root)

    if env['CM_MLPERF_DEVICE'] == 'gpu':
        env['+C_INCLUDE_PATH'].append(env['CM_CUDA_PATH_INCLUDE'])
        env['+CPLUS_INCLUDE_PATH'].append(env['CM_CUDA_PATH_INCLUDE'])
        env['+LD_LIBRARY_PATH'].append(env['CM_CUDA_PATH_LIB'])
        env['+DYLD_FALLBACK_LIBRARY_PATH'].append(env['CM_CUDA_PATH_INCLUDE'])

    elif env['CM_MLPERF_DEVICE'] == 'qaic':
        source_files.append(
            os.path.join(
                kilt_root,
                "devices",
                "qaic",
                "api",
                "master",
                "QAicInfApi.cpp"))

    print(f"Compiling the source files: {source_files}")
    env['CM_CXX_SOURCE_FILES'] = ";".join(source_files)

    env['+ CXXFLAGS'].append("-std=c++17")
    env['+ CXXFLAGS'].append("-fpermissive")

    env['+ CXXFLAGS'].append("-DKILT_CONFIG_FROM_ENV")
    env['+ CXXFLAGS'].append("-DKILT_CONFIG_TRANSLATE_X")
    env['+ CXXFLAGS'].append("-DKILT_BENCHMARK_" + env['CM_BENCHMARK'])
    env['+ CXXFLAGS'].append("-DKILT_DEVICE_" + env['device'].upper())

    # add preprocessor flag like "#define CM_MODEL_RESNET50"
    # env['+ CXXFLAGS'].append('-DCM_MODEL_' + env['CM_MODEL'].upper())
    # add preprocessor flag like "#define CM_MLPERF_BACKEND_ONNXRUNTIME"
    env['+ CXXFLAGS'].append('-DCM_MLPERF_BACKEND_' +
                             env['CM_MLPERF_BACKEND'].upper())
    # add preprocessor flag like "#define CM_MLPERF_DEVICE_CPU"
    env['+ CXXFLAGS'].append('-DCM_MLPERF_DEVICE_' +
                             env['CM_MLPERF_DEVICE'].upper())

    if '+ LDCXXFLAGS' not in env:
        env['+ LDCXXFLAGS'] = []

    env['+ LDCXXFLAGS'] += [
        "-lmlperf_loadgen",
        "-lpthread",
        "-ldl"
    ]
    # e.g. -lonnxruntime
    if 'CM_MLPERF_BACKEND_LIB_NAMESPEC' in env:
        env['+ LDCXXFLAGS'].append('-l' +
                                   env['CM_MLPERF_BACKEND_LIB_NAMESPEC'])
    # e.g. -lcudart
    if 'CM_MLPERF_DEVICE_LIB_NAMESPEC' in env:
        env['+ LDCXXFLAGS'].append('-l' + env['CM_MLPERF_DEVICE_LIB_NAMESPEC'])

    if '-DPRINT_NETWORK_DESCRIPTOR' in env['+ CXXFLAGS']:
        env['+ LDCXXFLAGS'].append('-lprotobuf')

    env['CM_LINKER_LANG'] = 'CXX'
    env['CM_RUN_DIR'] = env.get('CM_MLPERF_OUTPUT_DIR', os.getcwd())

    if 'CM_MLPERF_CONF' not in env:
        env['CM_MLPERF_CONF'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "mlperf.conf")
    if 'CM_MLPERF_USER_CONF' not in env:
        env['CM_MLPERF_USER_CONF'] = os.path.join(
            env['CM_MLPERF_INFERENCE_CLASSIFICATION_AND_DETECTION_PATH'], "user.conf")

    # to LOADGEN_MLPERF_CONF
    env['loadgen_mlperf_conf_path'] = env['CM_MLPERF_CONF']
    # to LOADGEN_USER_CONF
    env['loadgen_user_conf_path'] = env['CM_MLPERF_USER_CONF']
    env['loadgen_scenario'] = env['CM_MLPERF_LOADGEN_SCENARIO']

    loadgen_mode = env['CM_MLPERF_LOADGEN_MODE']
    if loadgen_mode == 'performance':
        kilt_loadgen_mode = 'PerformanceOnly'
    elif loadgen_mode == 'accuracy':
        kilt_loadgen_mode = 'AccuracyOnly'
    elif loadgen_mode == 'compliance':
        kilt_loadgen_mode = 'PerformanceOnly'
    else:
        return {'return': 1, 'error': 'Unknown loadgen mode'}
    env['loadgen_mode'] = kilt_loadgen_mode

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import subprocess


def preprocess(i):

    env = i['env']

    if str(env.get('CM_DETECT_USING_PYCUDA', '')
           ).lower() in ["1", "yes", "true"]:
        i['run_script_input']['script_name'] = 'detect'

    return {'return': 0}


def postprocess(i):

    env = i['env']
    state = i['state']

    os_info = i['os_info']

    r = utils.load_txt(file_name='tmp-run.out',
                       check_if_exists=True,
                       split=True)
    if r['return'] > 0:
        return r

    lst = r['list']

    # properties
    p = {}
    gpu = {}

    gpu_id = -1

    for line in lst:
        # print (line)

        j = line.find(':')

        if j >= 0:
            key = line[:j].strip()
            val = line[j + 1:].strip()

            if key == "GPU Device ID":
                gpu_id += 1
                gpu[gpu_id] = {}

            if gpu_id < 0:
                continue

            gpu[gpu_id][key] = val
            p[key] = val

            key_env = 'CM_CUDA_DEVICE_PROP_' + key.upper().replace(' ', '_')
            env[key_env] = val

    state['cm_cuda_num_devices'] = gpu_id + 1
    env['CM_CUDA_NUM_DEVICES'] = gpu_id + 1

    state['cm_cuda_device_prop'] = p
    state['cm_cuda_devices_prop'] = gpu

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import cmind as cm
import os
import configparser


def preprocess(i):

    os_info = i['os_info']
    env = i['env']

    if not env['CM_MLPERF_RUN_CMD']:
        env['CM_MLPERF_RUN_CMD'] = os.path.join(
            i['run_script_input']['path'], "dummy.sh")

    if 'CM_MLPERF_POWER_TIMESTAMP' in env:
        timestamp = ""
    else:
        timestamp = " --no-timestamp-path"

    if 'CM_MLPERF_LOADGEN_LOGS_DIR' not in env:
        env['CM_MLPERF_LOADGEN_LOGS_DIR'] = os.path.join(
            os.getcwd(), "loadgen_logs")

    run_cmd = env['CM_MLPERF_RUN_CMD'].replace("'", '"')
    run_cmd = run_cmd.replace('"', '\\"')
    cmd = env['CM_PYTHON_BIN_WITH_PATH'] + ' ' +\
        os.path.join(env['CM_MLPERF_POWER_SOURCE'], 'ptd_client_server', 'client.py') + \
        " -a " + env['CM_MLPERF_POWER_SERVER_ADDRESS'] + \
        " -p " + env.get('CM_MLPERF_POWER_SERVER_PORT', "4950") + \
        " -w '" + run_cmd + \
        "' -L " + env['CM_MLPERF_LOADGEN_LOGS_DIR'] + \
        " -o " + env['CM_MLPERF_POWER_LOG_DIR'] + \
        " -n " + env['CM_MLPERF_POWER_NTP_SERVER'] + \
        timestamp

    if 'CM_MLPERF_POWER_MAX_AMPS' in env and 'CM_MLPERF_POWER_MAX_VOLTS' in env:
        cmd = cmd + " --max-amps " + env['CM_MLPERF_POWER_MAX_AMPS'] + \
            " --max-volts " + env['CM_MLPERF_POWER_MAX_VOLTS']

    env['CM_MLPERF_POWER_RUN_CMD'] = cmd

    return {'return': 0}


def postprocess(i):
    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    if env.get('CM_QAIC_ECC', '') == 'yes':
        import json
        for device in env['CM_QAIC_DEVICES'].split(","):
            ecc_template = {}
            ecc_template['request'] = []
            ecc_template['request'].append({})
            ecc_template['request'][0]['qid'] = device
            ecc_template['request'][0]['dev_config'] = {}
            ecc_template['request'][0]['dev_config']['update_ras_ecc_config_request'] = {
            }
            ecc_template['request'][0]['dev_config']['update_ras_ecc_config_request']['ras_ecc'] = [
            ]
            ecc_template['request'][0]['dev_config']['update_ras_ecc_config_request']['ras_ecc'].append(
                "RAS_DDR_ECC")
            with open("request_" + device + ".json", "w") as f:
                f.write(json.dumps(ecc_template))

    if env.get('CM_QAIC_VC', '') != '':
        env['CM_QAIC_VC_HEX'] = hex(int(env['CM_QAIC_VC']))

    return {'return': 0}


def postprocess(i):

    env = i['env']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os
import json
import shutil
import subprocess


def preprocess(i):

    os_info = i['os_info']
    env = i['env']
    state = i['state']
    script_path = i['run_script_input']['path']

    if env.get('CM_MLPERF_SKIP_RUN', '') == "yes":
        return {'return': 0}

    if env.get('CM_RUN_DOCKER_CONTAINER', '') == "yes":
        return {'return': 0}

    if env.get('CM_MLPERF_POWER', '') == "yes":
        power = "yes"
    else:
        power = "no"

    rerun = True if env.get("CM_RERUN", "") != '' else False

    if 'CM_MLPERF_LOADGEN_SCENARIO' not in env:
        env['CM_MLPERF_LOADGEN_SCENARIO'] = "Offline"

    if 'CM_MLPERF_LOADGEN_MODE' not in env:
        env['CM_MLPERF_LOADGEN_MODE'] = "accuracy"

    if 'CM_MODEL' not in env:
        return {
            'return': 1, 'error': "Please select a variation specifying the model to run"}

    # if env['CM_MODEL'] == "resnet50":
    #    cmd = "cp " + os.path.join(env['CM_DATASET_AUX_PATH'], "val.txt") + " " + os.path.join(env['CM_DATASET_PATH'],
    #    "val_map.txt")
    #    ret = os.system(cmd)

    env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] = " " + \
        env.get('CM_MLPERF_LOADGEN_EXTRA_OPTIONS', '') + " "

    if 'CM_MLPERF_LOADGEN_QPS' not in env:
        env['CM_MLPERF_LOADGEN_QPS_OPT'] = ""
    else:
        env['CM_MLPERF_LOADGEN_QPS_OPT'] = " --qps " + \
            env['CM_MLPERF_LOADGEN_QPS']

    env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += env['CM_MLPERF_LOADGEN_QPS_OPT']

    if 'CM_NUM_THREADS' not in env:
        if 'CM_MINIMIZE_THREADS' in env:
            env['CM_NUM_THREADS'] = str(int(env['CM_HOST_CPU_TOTAL_CORES']) //
                                        (int(env.get('CM_HOST_CPU_SOCKETS', '1')) * int(env.get('CM_HOST_CPU_TOTAL_CORES', '1'))))
        else:
            env['CM_NUM_THREADS'] = env.get('CM_HOST_CPU_TOTAL_CORES', '1')

    if env.get('CM_MLPERF_LOADGEN_MAX_BATCHSIZE', '') != '' and str(env.get(
            'CM_MLPERF_MODEL_SKIP_BATCHING', False)).lower() not in ["true", "1", "yes"]:
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --max-batchsize " + \
            str(env['CM_MLPERF_LOADGEN_MAX_BATCHSIZE'])

    if env.get('CM_MLPERF_LOADGEN_BATCH_SIZE', '') != '':
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --batch-size " + \
            str(env['CM_MLPERF_LOADGEN_BATCH_SIZE'])

    if env.get('CM_MLPERF_LOADGEN_QUERY_COUNT', '') != '' and not env.get('CM_TMP_IGNORE_MLPERF_QUERY_COUNT', False) and (
            env['CM_MLPERF_LOADGEN_MODE'] == 'accuracy' or 'gptj' in env['CM_MODEL'] or 'llama2' in env['CM_MODEL'] or 'mixtral' in env['CM_MODEL'] or 'llama3' in env['CM_MODEL']) and env.get('CM_MLPERF_RUN_STYLE', '') != "valid":
        env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --count " + \
            env['CM_MLPERF_LOADGEN_QUERY_COUNT']

    print("Using MLCommons Inference source from '" +
          env['CM_MLPERF_INFERENCE_SOURCE'] + "'")

    if 'CM_MLPERF_CONF' not in env:
        env['CM_MLPERF_CONF'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "mlperf.conf")

    x = "" if os_info['platform'] == 'windows' else "'"

    inference_src_version = env.get('CM_MLPERF_INFERENCE_SOURCE_VERSION', '')
    version_tuple = None
    if inference_src_version:
        version_tuple = tuple(map(int, inference_src_version.split('.')))

    if version_tuple and version_tuple >= (4, 1, 1):
        pass  # mlperf_conf is automatically loaded by the loadgen
    else:
        if "llama2-70b" in env['CM_MODEL'] or "mixtral-8x7b" in env["CM_MODEL"]:
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --mlperf-conf " + \
                x + env['CM_MLPERF_CONF'] + x
        else:
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] += " --mlperf_conf " + \
                x + env['CM_MLPERF_CONF'] + x

    if env.get('CM_NETWORK_LOADGEN', '') != "lon" and env.get(
            'CM_MLPERF_INFERENCE_API_SERVER', '') == '' and "llama2-70b" not in env['CM_MODEL']:
        env['MODEL_DIR'] = env.get('CM_ML_MODEL_PATH')
        if not env['MODEL_DIR']:
            env['MODEL_DIR'] = os.path.dirname(
                env.get(
                    'CM_MLPERF_CUSTOM_MODEL_PATH',
                    env.get(
                        'CM_ML_MODEL_FILE_WITH_PATH',
                        '')))

    RUN_CMD = ""
    state['RUN'] = {}

    scenario = env['CM_MLPERF_LOADGEN_SCENARIO']
    state['RUN'][scenario] = {}
    scenario_extra_options = ''

    NUM_THREADS = env['CM_NUM_THREADS']
    if int(
            NUM_THREADS) > 2 and env['CM_MLPERF_DEVICE'] == "gpu" and env['CM_MODEL'] != "rgat":
        NUM_THREADS = "2"  # Don't use more than 2 threads when run on GPU

    if env['CM_MODEL'] in ['resnet50', 'retinanet',
                           'stable-diffusion-xl', 'rgat']:
        scenario_extra_options += " --threads " + NUM_THREADS

    ml_model_name = env['CM_MODEL']
    if 'CM_MLPERF_USER_CONF' in env:
        user_conf_path = env['CM_MLPERF_USER_CONF']
        x = "" if os_info['platform'] == 'windows' else "'"
        if 'llama2-70b' in env['CM_MODEL'] or "mixtral-8x7b" in env["CM_MODEL"] or "llama3" in env["CM_MODEL"]:
            scenario_extra_options += " --user-conf " + x + user_conf_path + x
        else:
            scenario_extra_options += " --user_conf " + x + user_conf_path + x

    mode = env['CM_MLPERF_LOADGEN_MODE']
    mode_extra_options = ""

    if 'CM_DATASET_PREPROCESSED_PATH' in env and env['CM_MODEL'] in [
            'resnet50', 'retinanet']:
        # dataset_options = " --use_preprocessed_dataset --preprocessed_dir "+env['CM_DATASET_PREPROCESSED_PATH']
        if env.get('CM_MLPERF_LAST_RELEASE') not in ["v2.0", "v2.1"]:
            dataset_options = " --use_preprocessed_dataset --cache_dir " + \
                env['CM_DATASET_PREPROCESSED_PATH']
        else:
            dataset_options = ""
        if env['CM_MODEL'] == "retinanet":
            dataset_options += " --dataset-list " + \
                env['CM_DATASET_ANNOTATIONS_FILE_PATH']
        elif env['CM_MODEL'] == "resnet50":
            dataset_options += " --dataset-list " + \
                os.path.join(env['CM_DATASET_AUX_PATH'], "val.txt")
        env['DATA_DIR'] = env.get('CM_DATASET_PREPROCESSED_PATH')
    else:
        if 'CM_DATASET_PREPROCESSED_PATH' in env:
            env['DATA_DIR'] = env.get('CM_DATASET_PREPROCESSED_PATH')
        else:
            env['DATA_DIR'] = env.get('CM_DATASET_PATH')

        if "dlrm" in env['CM_MODEL']:
            env['DATA_DIR'] = env['CM_CRITEO_PREPROCESSED_PATH']

        dataset_options = ''

    if env.get('CM_MLPERF_EXTRA_DATASET_ARGS', '') != '':
        dataset_options += " " + env['CM_MLPERF_EXTRA_DATASET_ARGS']

    if mode == "accuracy":
        mode_extra_options += " --accuracy"

    elif mode == "performance":
        pass

    elif mode == "compliance":

        audit_full_path = env['CM_MLPERF_INFERENCE_AUDIT_PATH']
        mode_extra_options = " --audit '" + audit_full_path + "'"

    if env.get('CM_MLPERF_OUTPUT_DIR', '') == '':
        env['CM_MLPERF_OUTPUT_DIR'] = os.getcwd()

    mlperf_implementation = env.get('CM_MLPERF_IMPLEMENTATION', 'reference')
    cmd, run_dir = get_run_cmd(os_info, env, scenario_extra_options,
                               mode_extra_options, dataset_options, mlperf_implementation)

    if env.get('CM_NETWORK_LOADGEN', '') == "lon":

        run_cmd = i['state']['mlperf_inference_run_cmd']
        env['CM_SSH_RUN_COMMANDS'] = []
        env['CM_SSH_RUN_COMMANDS'].append(
            run_cmd.replace(
                "--network=lon",
                "--network=sut") + " &")

    env['CM_MLPERF_RUN_CMD'] = cmd
    env['CM_RUN_DIR'] = run_dir
    env['CM_RUN_CMD'] = cmd
    env['CK_PROGRAM_TMP_DIR'] = env.get('CM_ML_MODEL_PATH')  # for tvm

    if env.get('CM_HOST_PLATFORM_FLAVOR', '') == "arm64":
        env['CM_HOST_PLATFORM_FLAVOR'] = "aarch64"

    return {'return': 0}


def get_run_cmd(os_info, env, scenario_extra_options,
                mode_extra_options, dataset_options, implementation="reference"):
    if implementation == "reference":
        return get_run_cmd_reference(
            os_info, env, scenario_extra_options, mode_extra_options, dataset_options)
    if implementation == "nvidia":
        return get_run_cmd_nvidia(
            os_info, env, scenario_extra_options, mode_extra_options, dataset_options)
    return "", os.getcwd()


def get_run_cmd_reference(
        os_info, env, scenario_extra_options, mode_extra_options, dataset_options):

    if env['CM_MODEL'] in ["gptj-99", "gptj-99.9"]:

        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "language", "gpt-j")
        if env.get('CM_NETWORK_LOADGEN', '') != "lon":
            cmd = env['CM_PYTHON_BIN_WITH_PATH'] +  \
                " main.py --model-path=" + env['CM_ML_MODEL_FILE_WITH_PATH'] + ' --dataset-path=' + env['CM_DATASET_EVAL_PATH'] + " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + " " + env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
                ' --dtype ' + env['CM_MLPERF_MODEL_PRECISION'] + \
                scenario_extra_options + mode_extra_options + dataset_options
        else:
            cmd = env['CM_PYTHON_BIN_WITH_PATH'] +  \
                " main.py" + ' --dataset-path=' + env['CM_DATASET_EVAL_PATH'] + " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + " " + env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
                ' --dtype ' + env['CM_MLPERF_MODEL_PRECISION'] + \
                scenario_extra_options + mode_extra_options + dataset_options
        cmd = cmd.replace("--count", "--max_examples")
        if env['CM_MLPERF_DEVICE'] == "gpu":
            gpu_options = " --gpu"
            env['CUDA_VISIBLE_DEVICES'] = "0"
        else:
            gpu_options = ""
        cmd = cmd + gpu_options
        env['LOG_PATH'] = env['CM_MLPERF_OUTPUT_DIR']

    if env['CM_MODEL'] in ["resnet50", "retinanet"]:

        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "vision",
            "classification_and_detection")
        env['OUTPUT_DIR'] = env['CM_MLPERF_OUTPUT_DIR']
        if env.get('CM_MLPERF_VISION_DATASET_OPTION', '') == '' and env.get(
                'CM_MLPERF_DEVICE') != "tpu":
            if os_info['platform'] == 'windows':
                cmd = "python python/main.py --profile " + env['CM_MODEL'] + "-" + env['CM_MLPERF_BACKEND'] + \
                    " --model=" + env['CM_ML_MODEL_FILE_WITH_PATH'] + ' --dataset-path=' + env['CM_DATASET_PREPROCESSED_PATH'] + \
                    " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + " " + \
                    " --output " + env['OUTPUT_DIR'] + " " + \
                    env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
                    scenario_extra_options + mode_extra_options + dataset_options
            else:
                cmd = "./run_local.sh " + env['CM_MLPERF_BACKEND'] + ' ' + \
                    env['CM_MODEL'] + ' ' + env['CM_MLPERF_DEVICE'] + " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + " " + env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
                    scenario_extra_options + mode_extra_options + dataset_options
            return cmd, env['RUN_DIR']

        if env['CM_MLPERF_BACKEND'] == "ncnn":
            env['MODEL_FILE'] = os.path.join(
                os.path.dirname(
                    env.get('CM_ML_MODEL_FILE_WITH_PATH')),
                "resnet50_v1")
        else:
            env['MODEL_FILE'] = env.get(
                'CM_MLPERF_CUSTOM_MODEL_PATH',
                env.get('CM_ML_MODEL_FILE_WITH_PATH'))
        if not env['MODEL_FILE']:
            return {'return': 1, 'error': 'No valid model file found!'}

        env['LOG_PATH'] = env['CM_MLPERF_OUTPUT_DIR']

        extra_options = " --output " + env['CM_MLPERF_OUTPUT_DIR'] + " --model-name resnet50  --dataset " + env['CM_MLPERF_VISION_DATASET_OPTION'] + ' --max-batchsize ' + env.get('CM_MLPERF_LOADGEN_MAX_BATCHSIZE', '1') + \
            " --dataset-path " + env['CM_DATASET_PREPROCESSED_PATH'] + " --model " + env['MODEL_FILE'] + \
            " --preprocessed_dir " + env['CM_DATASET_PREPROCESSED_PATH']

        if env.get('CM_MLPERF_DEVICE') == "tpu":
            cmd = "cd '" + os.path.join(env['RUN_DIR'], "python") + "' && " + env.get('CM_SUDO', "") + " " + env['CM_PYTHON_BIN_WITH_PATH'] + " main.py " +\
                "--backend " + env['CM_MLPERF_BACKEND'] + " --scenario=" + env['CM_MLPERF_LOADGEN_SCENARIO'] + " --device tpu " + \
                env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + scenario_extra_options + \
                mode_extra_options + dataset_options + extra_options
        else:
            cmd = "cd '" + os.path.join(env['RUN_DIR'], "python") + "' && " + env['CM_PYTHON_BIN_WITH_PATH'] + " main.py " +\
                "--backend " + env['CM_MLPERF_BACKEND'] + " --scenario=" + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
                env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + scenario_extra_options + \
                mode_extra_options + dataset_options + extra_options
        env['SKIP_VERIFY_ACCURACY'] = True

    elif "bert" in env['CM_MODEL']:

        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "language", "bert")
        env['MODEL_FILE'] = env.get(
            'CM_MLPERF_CUSTOM_MODEL_PATH',
            env.get('CM_ML_MODEL_FILE_WITH_PATH'))
        if not env['MODEL_FILE']:
            return {'return': 1, 'error': 'No valid model file found!'}
        if env.get('CM_MLPERF_QUANTIZATION') in ["on", True, "1", "True"]:
            quantization_options = " --quantized"
        else:
            quantization_options = ""
        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " run.py --backend=" + env['CM_MLPERF_BACKEND'] + " --scenario=" + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + scenario_extra_options + \
            mode_extra_options + dataset_options + quantization_options
        if env['CM_MLPERF_BACKEND'] == "deepsparse":
            cmd += " --batch_size=" + \
                env.get('CM_MLPERF_LOADGEN_MAX_BATCHSIZE', '1') + \
                " --model_path=" + env['MODEL_FILE']

        if env.get('CM_MLPERF_CUSTOM_MODEL_PATH', '') != '':
            env['CM_ML_MODEL_FILE_WITH_PATH'] = env['MODEL_FILE']

        cmd = cmd.replace("--count", "--max_examples")
        env['VOCAB_FILE'] = env['CM_ML_MODEL_BERT_VOCAB_FILE_WITH_PATH']
        env['DATASET_FILE'] = env['CM_DATASET_SQUAD_VAL_PATH']
        env['LOG_PATH'] = env['CM_MLPERF_OUTPUT_DIR']
        env['SKIP_VERIFY_ACCURACY'] = True

    elif "rnnt" in env['CM_MODEL']:

        env['RUN_DIR'] = env['CM_MLPERF_INFERENCE_RNNT_PATH']
        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " run.py --backend " + env['CM_MLPERF_BACKEND'] + \
            " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            " --manifest " + env['CM_DATASET_PREPROCESSED_JSON'] + \
            " --dataset_dir " + os.path.join(env['CM_DATASET_PREPROCESSED_PATH'], "..") + \
            " --pytorch_config_toml " + os.path.join("pytorch", "configs", "rnnt.toml") + \
            " --pytorch_checkpoint " + env['CM_ML_MODEL_FILE_WITH_PATH'] + \
            " --log_dir " + env['CM_MLPERF_OUTPUT_DIR'] + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            scenario_extra_options + mode_extra_options + dataset_options
        env['SKIP_VERIFY_ACCURACY'] = True

    elif "stable-diffusion-xl" in env['CM_MODEL']:
        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'], "text_to_image")
        if env.get('+PYTHONPATH', '') == '':
            env['+PYTHONPATH'] = []
        env['+PYTHONPATH'].append(
            os.path.join(
                env['CM_MLPERF_INFERENCE_SOURCE'],
                "text_to_image",
                "tools",
                "fid"))

        backend = env['CM_MLPERF_BACKEND']
        device = env['CM_MLPERF_DEVICE'] if env['CM_MLPERF_DEVICE'] not in [
            "gpu", "rocm"] else "cuda"
        max_batchsize = env.get('CM_MLPERF_LOADGEN_MAX_BATCHSIZE', '1')
        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " main.py " \
            " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            " --profile " + 'stable-diffusion-xl-pytorch ' + \
            " --dataset " + 'coco-1024' +  \
            " --dataset-path " + env['CM_DATASET_PATH_ROOT'] + \
            ' --dtype ' + env['CM_MLPERF_MODEL_PRECISION'].replace("bfloat", "bf").replace("float", "fp") + \
            " --device " + device + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            scenario_extra_options + mode_extra_options + \
            " --output " + env['CM_MLPERF_OUTPUT_DIR'] + \
            " --model-path " + env['CM_ML_MODEL_PATH']
        if "--max-batchsize" not in cmd:
            cmd += " --max-batchsize " + max_batchsize
        if env.get('CM_COCO2014_SAMPLE_ID_PATH', '') != '':
            cmd += " --ids-path " + env['CM_COCO2014_SAMPLE_ID_PATH']

    elif "llama2-70b" in env['CM_MODEL']:
        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "language",
            "llama2-70b")
        backend = env['CM_MLPERF_BACKEND']
        device = env['CM_MLPERF_DEVICE'] if env['CM_MLPERF_DEVICE'] != "gpu" else "cuda"

        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " main.py " \
            " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            " --dataset-path " + env['CM_DATASET_PREPROCESSED_PATH'] + \
            " --device " + device.replace("cuda", "cuda:0") + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            scenario_extra_options + mode_extra_options + \
            " --output-log-dir " + env['CM_MLPERF_OUTPUT_DIR'] + \
            ' --dtype ' + env['CM_MLPERF_MODEL_PRECISION']

        if env.get('CM_MLPERF_INFERENCE_API_SERVER', '') != '':
            env['CM_VLLM_SERVER_MODEL_NAME'] = env.get(
                "CM_VLLM_SERVER_MODEL_NAME") or "NousResearch/Meta-Llama-3-8B-Instruct"
            # env['CM_MLPERF_INFERENCE_API_SERVER'] = "http://localhost:8000"
            cmd += f""" --api-server {env['CM_MLPERF_INFERENCE_API_SERVER']} \
                    --model-path {env['CM_VLLM_SERVER_MODEL_NAME']} \
                    --api-model-name {env['CM_VLLM_SERVER_MODEL_NAME']} --vllm """
        else:
            cmd += f" --model-path {env['LLAMA2_CHECKPOINT_PATH']}"

        if env.get('CM_MLPERF_INFERENCE_NUM_WORKERS', '') != '':
            cmd += f" --num-workers {env['CM_MLPERF_INFERENCE_NUM_WORKERS']}"

        cmd = cmd.replace("--count", "--total-sample-count")
        cmd = cmd.replace("--max-batchsize", "--batch-size")

    elif "mixtral-8x7b" in env['CM_MODEL']:
        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "language",
            "mixtral-8x7b")
        backend = env['CM_MLPERF_BACKEND']
        device = env['CM_MLPERF_DEVICE'] if env['CM_MLPERF_DEVICE'] != "gpu" else "cuda"
        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " main.py " \
            " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            " --dataset-path " + env['CM_DATASET_MIXTRAL_PREPROCESSED_PATH'] + \
            " --device " + device.replace("cuda", "cuda:0") + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            scenario_extra_options + mode_extra_options + \
            " --output-log-dir " + env['CM_MLPERF_OUTPUT_DIR'] + \
            ' --dtype ' + env['CM_MLPERF_MODEL_PRECISION'] + \
            " --model-path " + env['MIXTRAL_CHECKPOINT_PATH']
        cmd = cmd.replace("--count", "--total-sample-count")
        cmd = cmd.replace("--max-batchsize", "--batch-size")

    elif "3d-unet" in env['CM_MODEL']:

        env['RUN_DIR'] = env['CM_MLPERF_INFERENCE_3DUNET_PATH']
        backend = env['CM_MLPERF_BACKEND'] if env['CM_MLPERF_BACKEND'] != 'tf' else 'tensorflow'
        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " run.py --backend=" + backend + " --scenario=" + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            " --model=" + env['CM_ML_MODEL_FILE_WITH_PATH'] + \
            " --preprocessed_data_dir=" + env['CM_DATASET_KITS19_PREPROCESSED_PATH'] + \
            scenario_extra_options + mode_extra_options + dataset_options

        env['LOG_PATH'] = env['CM_MLPERF_OUTPUT_DIR']
        env['SKIP_VERIFY_ACCURACY'] = True

    elif "dlrm" in env['CM_MODEL']:  # DLRM is in draft stage

        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_DLRM_V2_PATH'], "pytorch")
        if 'multihot-criteo-sample' in env['CM_ML_MODEL_DATASET_TYPE']:
            dataset = "multihot-criteo-sample"
        elif 'multihot-criteo' in env['CM_ML_MODEL_DATASET_TYPE']:
            dataset = "multihot-criteo"

        env['MODEL_DIR'] = os.path.join(env['MODEL_DIR'], "model_weights")

        if env.get('CM_MLPERF_BIN_LOADER', '') == 'yes':
            mlperf_bin_loader_string = " --mlperf-bin-loader"
        else:
            mlperf_bin_loader_string = ""
        if env.get('CM_ML_MODEL_DEBUG', '') == 'yes':
            config = " --max-ind-range=10000000 --data-sub-sample-rate=0.875 "
        else:
            config = "  --max-ind-range=40000000 "

        if env['CM_MLPERF_DEVICE'] == "gpu":
            gpu_options = ""
            env['CUDA_VISIBLE_DEVICES'] = "0"
        else:
            gpu_options = ""
            env['WORLD_SIZE'] = "1"

        if env['CM_MLPERF_LOADGEN_MODE'] == "accuracy" and env['CM_MLPERF_LOADGEN_SCENARIO'] == "Offline":
            mode_extra_options += " --samples-per-query-offline=1"

        cmd = " ./run_local.sh " + env['CM_MLPERF_BACKEND'] + \
            ' dlrm ' + dataset + ' ' + env['CM_MLPERF_DEVICE'] + " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + " " + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            config + mlperf_bin_loader_string + \
            ' --samples-to-aggregate-quantile-file=./tools/dist_quantile.txt ' + \
            scenario_extra_options + mode_extra_options + dataset_options + gpu_options
        cmd = cmd.replace("--count", "--count-queries")
        env['OUTPUT_DIR'] = env['CM_MLPERF_OUTPUT_DIR']

    elif "rgat" in env['CM_MODEL']:
        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "graph",
            "R-GAT")
        backend = env['CM_MLPERF_BACKEND']

        dtype_rgat = env['CM_MLPERF_MODEL_PRECISION'].replace("float", "fp")

        if env.get('CM_MLPERF_SUBMISSION_GENERATION_STYLE', '') == "full":
            mode_extra_options += " --dataset igbh-dgl --profile rgat-dgl-full "
        else:
            mode_extra_options += " --dataset igbh-dgl-tiny --profile debug-dgl "

        device = env['CM_MLPERF_DEVICE'] if env['CM_MLPERF_DEVICE'] != "gpu" else "cuda"
        # have to add the condition for running in debug mode or real run mode
        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " main.py " \
            " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            " --dataset-path " + env['CM_DATASET_IGBH_PATH'] + \
            " --device " + device.replace("cuda", "gpu") + \
            env['CM_MLPERF_LOADGEN_EXTRA_OPTIONS'] + \
            scenario_extra_options + mode_extra_options + \
            " --output " + env['CM_MLPERF_OUTPUT_DIR'] + \
            ' --dtype ' + dtype_rgat + \
            " --model-path " + env['RGAT_CHECKPOINT_PATH']

        if env.get('CM_ACTIVATE_RGAT_IN_MEMORY', '') == "yes":
            cmd += " --in-memory "

    elif "llama3" in env['CM_MODEL']:
        env['RUN_DIR'] = os.path.join(
            env['CM_MLPERF_INFERENCE_SOURCE'],
            "language",
            "llama3.1-405b")

        if int(env.get('CM_MLPERF_INFERENCE_TP_SIZE', '')) > 1:
            env['VLLM_WORKER_MULTIPROC_METHOD'] = "spawn"

        cmd = env['CM_PYTHON_BIN_WITH_PATH'] + " main.py " \
            " --scenario " + env['CM_MLPERF_LOADGEN_SCENARIO'] + \
            " --dataset-path " + env['CM_DATASET_LLAMA3_PATH'] + \
            " --output-log-dir " + env['CM_MLPERF_OUTPUT_DIR'] + \
            ' --dtype ' + env['CM_MLPERF_MODEL_PRECISION'] + \
            " --model-path " + env['CM_ML_MODEL_LLAMA3_CHECKPOINT_PATH'] + \
            " --tensor-parallel-size " + env['CM_MLPERF_INFERENCE_TP_SIZE'] + \
            " --vllm "

        if env.get('CM_MLPERF_INFERENCE_NUM_WORKERS', '') != '':
            cmd += f" --num-workers {env['CM_MLPERF_INFERENCE_NUM_WORKERS']}"

        cmd = cmd.replace("--count", "--total-sample-count")
        cmd = cmd.replace("--max-batchsize", "--batch-size")

    if env.get('CM_NETWORK_LOADGEN', '') in ["lon", "sut"]:
        cmd = cmd + " " + "--network " + env['CM_NETWORK_LOADGEN']
        if env.get('CM_NETWORK_LOADGEN_SUT_SERVERS', []):
            sut_servers = env['CM_NETWORK_LOADGEN_SUT_SERVERS']
            cmd += " --sut_server '" + "','".join(sut_servers) + "' "

    return cmd, env['RUN_DIR']


def postprocess(i):

    env = i['env']
    state = i['state']

    inp = i['input']

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    env = i['env']

    meta = i['meta']

    automation = i['automation']

    env['CM_QAIC_SOFTWARE_KIT_PATH'] = env['CM_GIT_CHECKOUT_PATH']

    quiet = (env.get('CM_QUIET', False) == 'yes')

    if env.get('+ CXXFLAGS', []) == []:
        env['+ CXXFLAGS'] = []
    if env.get('+ CFLAGS', []) == []:
        env['+ CFLAGS'] = []

    if env.get('CM_LLVM_CLANG_VERSION', '') != '':
        clang_version_split = env['CM_LLVM_CLANG_VERSION'].split(".")
        clang_major_version = int(clang_version_split[0])

        if clang_major_version >= 17:
            env['+ CFLAGS'].append("-Wno-error=c2x-extensions")

        if clang_major_version >= 16:
            env['+ CFLAGS'].append("-Wno-error=unused-but-set-variable")
            env['+ CXXFLAGS'].append("-Wno-error=unused-but-set-variable")

        if clang_major_version >= 13:
            env['+ CFLAGS'].append("-Wno-error=unused-const-variable")
            env['+ CFLAGS'].append("-Wno-error=unused-but-set-variable")
            env['+ CFLAGS'].append("-Wno-error=strict-prototypes")
            env['+ CFLAGS'].append("-Wno-error=unused-variable")
            env['+ CXXFLAGS'].append("-Wno-error=unused-const-variable")
            env['+ CXXFLAGS'].append("-Wno-error=unused-variable")
            env['+ CXXFLAGS'].append("-Wno-error=unused-private-field")
            env['+ CXXFLAGS'].append("-Wno-error=unused-result")
            env['+ CXXFLAGS'].append("-Wno-error=string-concatenation")
            env['+ CXXFLAGS'].append("-Wno-error=infinite-recursion")

        if clang_major_version == 12:
            env['+ CXXFLAGS'].append("-Wno-error=unknown-warning-option")

    return {'return': 0}


def postprocess(i):

    env = i['env']
    env['CM_QAIC_RUNNER_PATH'] = os.path.join(
        env['CM_QAIC_SOFTWARE_KIT_PATH'], "build", "utils", "qaic-runner")

    if '+PATH' not in env:
        env['+PATH'] = []

    env['+PATH'].append(env['CM_QAIC_RUNNER_PATH'])
    env['CM_QAIC_RUNNER_PATH'] = os.path.join(
        env['CM_QAIC_RUNNER_PATH'], "qaic-runner")

    return {'return': 0}



#
# Copyright: https://github.com/mlcommons/ck/blob/master/cm-mlops/COPYRIGHT.md
# License: https://github.com/mlcommons/ck/blob/master/cm-mlops/LICENSE.md
#
# White paper: https://arxiv.org/abs/2406.16791
# History: https://github.com/mlcommons/ck/blob/master/HISTORY.CM.md
# Original repository: https://github.com/mlcommons/ck/tree/master/cm-mlops
#
# CK and CM project contributors: https://github.com/mlcommons/ck/blob/master/CONTRIBUTING.md
#

from cmind import utils
import os


def preprocess(i):

    os_info = i['os_info']

    if os_info['platform'] == 'windows':
        return {'return': 1, 'error': 'Windows is not supported in this script yet'}

    env = i['env']

    automation = i['automation']

    recursion_spaces = i['recursion_spaces']

    need_version = env.get('CM_VERSION', '')
    if need_version == '':
        return {'return': 1,
                'error': 'internal problem - CM_VERSION is not defined in env'}

    print(recursion_spaces + '    # Requested version: {}'.format(need_version))

    return {'return': 0}


def postprocess(i):
    inp = i['input']
    env = i['env']
    tags = inp['tags']
    tag_list = tags.split(",")
    install_path = os.path.join(
        os.getcwd(),
        'openssl-' +
        env['CM_VERSION'] +
        'g',
        'install')
    path_lib = os.path.join(install_path, 'lib')
    if '+LD_LIBRARY_PATH' not in env:
        env['+LD_LIBRARY_PATH'] = []
    env['+LD_LIBRARY_PATH'].append(path_lib)
    bin_name = "openssl"
    path_bin = os.path.join(install_path, 'bin')
    env['CM_OPENSSL_INSTALLED_PATH'] = path_bin
    env['CM_OPENSSL_BIN_WITH_PATH'] = os.path.join(path_bin, bin_name)
    return {'return': 0}
